/*
***************************************************************************
**              ROCKWELL CONFIDENTIAL AND PROPRIETARY
**        This source is the sole property of ROCKWELL Inc.
**    Reproduction or utilization of this source in whole or in
**  part is forbidden without the written consent of ROCKWELL Inc.
***************************************************************************
**                 (c) Copyright ROCKWELL, INC. 1991
**                       All Rights Reserved
***************************************************************************
*/

/*
**      Last PVCS update: $Date:   01 Oct 1992  8:06:44  $
*/

/*
**      TITLE:          EXAMPLE1.c
**
**      FACILITY:       
**
**      MODULE DESCRIPTION:
**
**      Source File for VAPI Example 1
**
*/

/*
**
**      Revision History:
**
**              $Log:   C:/api_lib/example1.c_v  $
 * 
 *    Rev 1.2   01 Oct 1992  8:06:44   Robert Chen
 * 
 *    Rev 1.1   09 Sep 1992 17:31:26   Robert Chen
 * 
 *    Rev 1.0   12 Aug 1992 19:56:08   Robert Chen
 * Initial revision.
 * 
 *    Rev 1.3   02 Jul 1992  9:41:24   Robert Chen
 *  
 * 
 *    Rev 1.2   24 Jun 1992 16:00:46   Robert Chen
 *  
 * 
 *    Rev 1.1   15 Jun 1992 10:54:56   Robert Chen
 *  
 * 
 *    Rev 1.0   26 May 1992 11:18:04   Robert Chen
 * Initial revision.
** 
**
***************************************************************************/

/*
** 
** The following is an example program that uses the VAPI 
** to record and playback a message from the handset.
** 
** To record a message:
** 
**       1) Pick up the handset
** 
**       2) Press '1' and start recording
** 
**       3) Press '4' to terminate the recording
**
**       4) Press '4' again to terminate the program
** 
** To retrieve a message:
** 
**       1) Pick up the handset
** 
**       2) Press '2' to retrieve a message
** 
**       3) Press '4' to terminate the  message or wait for 
**          the playback to cmplete
**
**       4) Press '4' again to terminate the program
** 
*/




#if defined( VF_SWIN ) || defined( VF_MWIN )
  #include <windows.h>
#endif

#include <string.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>


#include "vapi.h"
#include "example.h"
// #include "vapisess.h"
#include "zapi.h"


#if defined(VF_SWIN) 
  #include "rawglobl.h"
  #include "raw.h"
  #include "rawvoice.h"
  #include "rawsdio.h"
  #include "rawerr.h"
  #include "rawclas1.h"
  #include "zapicls1.h"
  
#endif

#define USR_TELCO       0
#define USR_HANDSET     1
#define USR_SPEAKER     2
#define USR_MICROPHONE  3
#define USR_HANDSPEAKER 4

#ifndef BOOL
#define BOOL int
#endif

struct USR_DATA USR;
static RVCALLBACK lpfnGenCallBack;
static BOOL bModemInUse;

#if defined(VF_DOS)

static void PrintTitle(char *);

void main(int argc, char *argv[])
{
   int argi; 

   USR.Port = 1;
   USR.Operation = USR_HANDSET;
   strcpy(USR.InitString, "#CLS=8#VSS=0#CLS=0");   // Set VSS to 0

   argi = 1; 

   while ((argc >= 2))
   {
          switch(argv[argi][1])
          {
            case 'i' :
               if (argv[argi+1] != NULL)
                  // Add Init str from command line
                  strcat(USR.InitString, argv[argi+1]);   
               break;
            case 'p' :
               if (argv[argi+1] != NULL)
                  sscanf(argv[argi+1], "%d", &USR.Port);
               break;
            case 'l' :
               if (argv[argi+1] != NULL)
                  sscanf(argv[argi+1], "%d", &USR.Operation);
               break;
            default :
               PrintTitle(argv[0]);
               exit(1);
            }
            argi += 2;
            argc -= 2;
   }

   if (RVOpenSess(RZRetIrqId()+USR.Port-1,0))
     exit(1);

  /*
   ** Open the Comp Port
   */
  if (RZOpenCom())
  {
    printf("Open Com%x Error\n",RZRetIrqId()+1);
    exit(1);
  }
  printf("\nDefault COM port - COM%x - 16550 %s\n",
         RZRetIrqId()+1,RZFifoPresent() ? "Present" : "Not Present");

   RVPollSysTimerInit();
   USR.ModemInUse = TRUE;
   USR.CurrentState = USR_STA_01;
   lpfnGenCallBack = RVCallBack;
  
   while (1)
   {
      // The following section of code can be inserted anywhere in the user's program.
      // However, it should be called frequently to guarrentee the proper
      // operation of the VAPIs
      if (USR.ModemInUse)
      {
         USR.ModemInUse = USR_HandSetOperations();
         RVPoll();
      }
      else 
         break;
    }
  if (RZCloseCom())
  {
    printf("Close Com%x Error\n",RZRetIrqId()+1);
    exit(1);
  }
  exit(0);
}

void PrintTitle(char *szCommandStr)
{
  printf("\n\n              VAPI Example Demonstrating Simple Voice Line Operations");
  printf(  "\n                               Rockwell International\n");
  printf(    "                                 All Rights Reserved\n");
  printf(    "                        Copyright 1992 Rockwell International\n\n");
  printf("Valid Switches:\n" );
  printf("  /{i, p, l} \n\n");
  printf("  i - Initialization string\n");
  printf("  p - Port \n");
  printf("  l - controls voice line selection as follows:\n");
  // printf("  {0,1,2,3,4} - controls voice line selection as follows:\n\n");
  printf("     0 - telco line with autoanswer (default)\n");
  printf("     1 - handset\n");
  printf("     2 - speaker\n");
  printf("     3 - microphone\n");
  printf("     4 - speaker/handset/telco line combination\n\n");
  printf("example -> /l 1 <activates the handset>\n");
}

#else 

// ---------------------------------------------------------------------------
// The Windows version of the "main" function appearing above, performing
// session initialization and end user initiated abort checking:
// ---------------------------------------------------------------------------
int VoiceHandSet(char chOperation, unsigned int wComPort)
{
  EXTDATA ExtData;
  MSG msg;

  // ------------------------------------------------------------------------
  // Select the voice line to correspond to the parameter passed.
  // ------------------------------------------------------------------------
  if (chOperation >= '0' && chOperation <= '4') 
    USR.Operation = chOperation - (char) '0';
  else
    return(1);

  // ------------------------------------------------------------------------
  // Initialize Extended Data Structure.
  // ------------------------------------------------------------------------
  memset(&ExtData, 0, sizeof(EXTDATA));

  if (UpdateExtData(&ExtData))
  {
    WRError(hWndMain, "UpdateExtData", IS_EXTDATANOMEM);
    return (1);
  }

  // ------------------------------------------------------------------------
  // Determine if user has requested a different COMx port and close original
  // session.
  // ------------------------------------------------------------------------
  if ( (iOldComPort != ( int ) wComPort) || !RVGetSessCnt())
  {
    if ((iOldComPort > -1) && RVGetSessCnt())
      RVCloseSess();

    if (RVOpenSess(wComPort, (LONG)(LPEXTDATA)&ExtData) != RV_STATUS_NONE)
      return (1);

    iOldComPort = wComPort;
  }

  if (RZOpenCom())
    return( 1 );

  // ------------------------------------------------------------------------
  // Change state of primary window to reflect impending transaction:
  // ------------------------------------------------------------------------
  EnableCancel(hWndMain);
  DisableMenuItems(hWndMain, TRUE, 1);

  // -------------------------------------------------------------------------
  // Initialize VAPI and create instance thunks for VAPI call back functions:
  // -------------------------------------------------------------------------
  if ((lpfnGenCallBack = MakeProcInstance(RVCallBack, hInst)) == NULL)
    return(1);

  // -------------------------------------------------------------------------
  // User's initialization to use the handset or telco line:
  // -------------------------------------------------------------------------
  RVPollSysTimerInit();
  USR.ModemInUse = TRUE;
  USR.Abort = FALSE;
  USR.CurrentState = USR_STA_01;

  // -------------------------------------------------------------------------
  // Select one of the two operations: handset or telco line:
  // -------------------------------------------------------------------------
  while (1)
  {
    if (PeekMessage((LPMSG)&msg, NULL, 0, 0, PM_REMOVE))
    {
      if (msg.message == WM_QUIT)
        break;
      if (!TranslateAccelerator(hWndMain, hAccelMain, &msg))
      {
        TranslateMessage((LPMSG)&msg);
        DispatchMessage((LPMSG)&msg);
      }
    }

    else if (USR.ModemInUse)
    {
      bModemInUse = USR.ModemInUse = USR_HandSetOperations();
      RVPoll();
    }

    else
      break;
  }

  FreeProcInstance(lpfnGenCallBack);

  // -------------------------------------------------------------------------
  // Restore menu options to accessible state:
  // -------------------------------------------------------------------------
  DisableCancel( hWndMain, FALSE );
  DisableMenuItems( hWndMain, FALSE, 1 );

  if (RZCloseCom())
  {
    wprintf("Close Com%x Error\n", wComPort+1);
    return(1);
  }
  return(0);
}

// --------------------------------------------------------------------------
// Check status of voice transaction.
// --------------------------------------------------------------------------
int EX1inProgress( void )
{
  return ( !bModemInUse ? 0 : USR.CurrentState );
}

/*
void SetEX1Last( void )
*/

#endif

int USR_HandSetOperations(void)
{
   // Return ModemInUse as TRUE unless exits from USR_STA_LAST.
   // User can also set the variable to FALSE in USR1_ErrHandler as part
   // of Error shutdown sequence.
   USR.ModemInUse = TRUE;

   switch (USR.CurrentState)
   {
      case USR_STA_01:
         // RVOpen(0, lpfnGenCallBack);
          RVOpen(USR.InitString, lpfnGenCallBack);
         USR.CurrentState = USR_STA_02;
         break;

      case USR_STA_02:
         switch(RVStatus())
         {
            case RV_STATUS_DONE:
               USR.CurrentState = USR_STA_SET_VM_BAUD_01;
               break;

            case RV_STATUS_NONE:
               break;

            default:
               USR1_ErrHandler();
               break;
         }
         break;

      case USR_STA_SET_VM_BAUD_01:
         // Set DCE to vocie mode and leave the baudrate along.
         RVSetVMBaudRate(8, -1);     
         USR.CurrentState= USR_STA_SET_VM_BAUD_02;        
         break;

      case USR_STA_SET_VM_BAUD_02:
         switch(RVStatus())
         {
            case RV_STATUS_DONE:
               USR.CurrentState = USR_STA_SET_SETTING_01;
               break;

            case RV_STATUS_NONE:
               break;

            default:
               USR1_ErrHandler();
               break;
         }
         break;

     case USR_STA_SET_SETTING_01:

         // Init Voice Parameter Struct
         USR.vps.BPS = 4;                  // 4 bits per sample
         USR.vps.SampleRate = 7200;        // Sample rate at 7.2Khz
         USR.vps.SilenceDeletion = 1;      // Enable Silence detection
         USR.vps.SilenceSensitivity = 0;   // 
         USR.vps.SilenceDetectPeriod = 55;    // Cause DCE to send a <DLE>q or <DLE>s
                                           // every 5.5sec during silence period 

         // Select the  voice line source: handset or telco line.
         USR.vps.VoiceLine = USR.Operation;

         // Init Voice Tone Detection Mask to all DTMFs during
         // playback, record and idle mode.
         USR.vtd.TxTD = 0x3f;
         USR.vtd.RxTD = 0x3f;
         USR.vtd.IdleTD = 0x3f;

         RVSetCurrentSetting(&USR.vps, &USR.vtd, 16);
         USR.CurrentState = USR_STA_SET_SETTING_02;
         break;

     case USR_STA_SET_SETTING_02:
         switch(RVStatus())
         {
            case RV_STATUS_DONE:
              switch (USR.Operation)
              {
                case USR_HANDSET:
                case USR_MICROPHONE:
                case USR_SPEAKER:
                  USR.CurrentState = USR_STA_VOICE_COMMAND;
                  break;

                case USR_TELCO:
                case USR_HANDSPEAKER:
                  USR.CurrentState = USR_STA_AUTOANSWER_01;
                  break;

                default:
                  USR1_ErrHandler();
               }
               break;

            case RV_STATUS_NONE:
               break;

            default:
               USR1_ErrHandler();
               break;
         }
         break;

     case USR_STA_AUTOANSWER_01:
         RVAnswer(0, 8, 16);
         USR.CurrentState = USR_STA_AUTOANSWER_02;
         break;

     case USR_STA_AUTOANSWER_02:
         switch(RVStatus())
         {
            case RV_STATUS_DONE:
               USR.CurrentState = USR_STA_VOICE_COMMAND;
               break;

            case RV_STATUS_TIMEOUT:
               USR.CurrentState = USR_STA_LAST;
               break;

            case RV_STATUS_NONE:
               break;

            default:
               USR1_ErrHandler();
               break;
         }
         break;

     case USR_STA_VOICE_COMMAND:
         switch(USR.Tone = USR1_GetTone())
         {
            // Uses DTMF * to record a message, DTMF 0 to playback a message
            // and 4 to terminate the handset operation.
            case '1' :
               // Use the Current Voice Parameter setting and Tone Detection Mask.
               // Let VAPI make an inquiry about the CMIdString.
               RVRecordFile(0, 0, 0, "GREETMSG.RIF");
               USR.CurrentState = USR_STA_VOICE_ONLINE_01;
               break;

            case '2' :
               // Use the Current Tone Detection Mask.
               // Let VAPI make an inquiry about the CMIdString.
               RVPlayFile(0, "GREETMSG.RIF");
               USR.CurrentState = USR_STA_VOICE_ONLINE_01;
               break;

            case '4' :
               RVStopVoiceIO(1);
               USR.CurrentState = USR_STA_LAST;
               break;

            default:
               break;
          }
          break;

     case USR_STA_VOICE_ONLINE_01:
         if (RVStatus() == RV_STATUS_DONE)
         {
            USR.CurrentState = USR_STA_VOICE_COMMAND;
            break;
         }

         switch(USR.Tone = USR1_GetTone())
         {
            // Uses DTMF 4  to stop record or playback of a message.
            case '4' :
            case 'b' :
               // Issue a stop command and wait in the same state
               // until the command is done.
               RVStopVoiceIO(0);
               USR.CurrentState = USR_STA_VOICE_ONLINE_01;
               break;
            default:
               break;
          }
          break;

    case USR_STA_LAST:
         switch(RVStatus())
         {
            case RV_STATUS_DONE:
               RVClose();
               USR.ModemInUse = FALSE;
               break;

            case RV_STATUS_NONE:
               break;

            default:
               USR1_ErrHandler();
               break;
         }
         break;

   default:
         USR.CurrentState = USR_STA_LAST;
         break;
      
   }

   #ifdef VF_DOS
   // Allow key abort in DOS 
   if (kbhit())
   {
     getch();
     // If a key is hit during voice TX or RX, terminate the 
     // current transfer operation. 
     if (USR.CurrentState == USR_STA_VOICE_ONLINE_01)
       RVStopVoiceIO(0);
     else 
     {
       // If a key is hit during any other states then issue the
       // Hangup command  then goto the last state 
       USR.CurrentState = USR_STA_LAST;
       RVStopVoiceIO(1);
     }
   }
   #else  // Window version's Abort
      if (bAbortReq &&                        // Some kind of abort happened
          !USR.Abort &&                       // and not abort from the CANCEL menu
          (USR.CurrentState != USR_STA_LAST)) // and not already here
      {
           USR.CurrentState = USR_STA_LAST;   // Exit gracefully
           RVStopVoiceIO(1);
      }
   #endif

   return(USR.ModemInUse);
}


char USR1_GetTone()
{
   // When the VAPI detects the DTMF tones from DCE, it calls the callback
   // function supplied by the user. The assumption here is that the callback
   // function stores the tone in USR.LastToneDetected. The tone is passed
   // to the callback function as an integer in the form of <DLE>code.

   char TmpTone;

   TmpTone = (char) (USR.ToneDetected & 0x00ff);    // Mask out the <DLE> 
   USR.ToneDetected = 0;                    // Reset the tone
   return(TmpTone);                              // Return the tone
}

void USR1_ErrHandler()
{
   USR.ModemInUse = FALSE;
   USR.Abort = TRUE;
#if defined(VF_SWIN) 
     bAbortReq = TRUE;
#endif

   // Plus any other shutdown code.
}

