/*
***************************************************************************
**              ROCKWELL CONFIDENTIAL AND PROPRIETARY
**        This source is the sole property of ROCKWELL Inc.
**    Reproduction or utilization of this source in whole or in 
**  part is forbidden without the written consent of ROCKWELL Inc.
***************************************************************************
**                 (c) Copyright ROCKWELL, INC. 1991
**                       All Rights Reserved
***************************************************************************
*/

/* 
** Last update:  04-29-92 (GCS)
**	Last PVCS update: $Date:   12 Aug 1992 20:05:26  $ 
*/

/*
**	TITLE:		API.H
**
**	FACILITY:	T30/T4 API
**
**	MODULE DESCRIPTION:
**
** Public portion of the system wide defines.
**
*/

/*
**
**	Revision History:
**
**		$Log:   D:/api_lib/fapi.h_v  $
   
      Rev 1.0   12 Aug 1992 20:05:26   Robert Chen
   Initial revision.
** 
**    Rev 1.11   29 Apr 1992 16:30:18   JIM
** Global update of T.30/T.4 API (FAPI) source for version 1.03.
** 
**    Rev 1.8   17 Oct 1991 10:39:10   JIM
** Global update of T.30/T.4 API source for version 1.00.
** 
**    Rev 1.7   12 Sep 1991 11:16:22   JIM
** Global update of T.30/T.4 API source for version 0.22B.
** 
**    Rev 1.6   13 Aug 1991 17:26:24   JIM
** Correction of API source check in to include new fax file source.
** 
**    Rev 1.5   08 Aug 1991 18:45:24   JIM
** Update to facilitate single session DLL version of API.
** 
**    Rev 1.0   11 Apr 1991 15:03:16   STAN
** Initial revision.
** 
*/
//
//  FAX definition
//
/*
** =======================================================================
**
** Defines and Types
**
** =======================================================================
*/

/*
** API Error Codes
*/
enum APIErrCodes
	{
	RF_STATUS_NONE=0,					/* No status */
	RF_STATUS_DCN_DONE,				/* All done, DCN sent out */
	RF_STATUS_GOTCAP,					/* Get modem capabilities */
	RF_STATUS_RESET_DONE,			/* Get modem capabilities */

	/*
	** Transmit Warnings
	*/
   RF_WARN_TX_T4_FALLBACK, 		/* Tx speed fallback request by T4 */
   RF_WARN_TX_CSI,					/* Tx CSI received */
	RF_WARN_TX_NEW_PAGE,				/* Starting a new page for Tx */
	RF_WARN_TX_END_PAGE,				/* Ending a page for Tx */

	/*
	** Received Warnings
	*/
	RF_WARN_RX_RING,	  				/* Ring detected */
	RF_WARN_RX_TSI,					/* Rx TSI frame received */
	RF_WARN_RX_NEW_PAGE,				/* Starting a new page for Rx */
	RF_WARN_RX_END_PAGE,				/* Ending a page for Rx */
	RF_WARN_RX_CIG,					/* Rx CIG frame received */
	RF_WARN_RX_DEL_FILE,				/* Deleting empty Rx file */
	RF_WARN_RX_LINE_ERROR,			/* Line Error */

	/*
	** Other Warnings
	*/
	RF_WARN_UNK_FRAME,				/* Unknown FCF in a HDLC frame */
	RF_WARN_GETCAP_SPEED,			/* Modem reports an unrecognized	speed */
	RF_WARN_BAD_HDLC_CHECKSUM,		/* HDLC frame received with bad checksum */
	RF_WARN_PHONE_OFFHOOK,			/* Phone going off hook */
	RF_WARN_PHONE_ONHOOK,			/* Phone going on hook */

	/*
	** T4 Warnings
	*/
	RF_WARN_T4_REQ_FALLBACK,		/* T4 request for fallback */

	/*
	** Tx File List, TSI, and Phone Numbers
	*/
	RF_ERR_FL_TELE_SIZE=-32767,	/* Phone number too big */
	RF_ERR_FL_PATH_SIZE,				/* Transmit path too big */
	RF_ERR_FL_ID_SIZE,		 		/* Bad TSI/CSI size (too long) */
	RF_ERR_FL_SPEED_RANGE,			/* User asked for Tx out of range speed */
	RF_ERR_FL_SPEED_MODEM,			/* User asked for Tx invalid modem speed */
	RF_ERR_FL_SPEED_DCS,	 			/* User asked for Tx invalid API speed */
	RF_ERR_FL_FORMAT,					/* User asked for Tx invalid format */
	RF_ERR_FL_MEDIA,					/* User specified unknown media */
	RF_ERR_FL_TIFF_BADFAX,			/* FIle error, Tx, verify file type */

	RF_ERR_FL_INVALID_ENTRY,		/* Invalid entry number */
	RF_ERR_FL_NO_SUCH_ENTRY,		/* No such entry in the Tx file list */
	RF_ERR_FL_BUFFER_PTR,			/* Invalid buffer ptr for BUFFER mode */
	RF_ERR_FL_BUFFER_SIZE,			/* Invalid buffer size for BUFFER mode */

	RF_ERR_FL_FILE_LIST_EMPTY,		/* Tx open with an empty file list */
	RF_ERR_FL_NO_DIAL_STRING,		/* No dial string for open Tx */
	RF_ERR_FL_FILE_LIST_DONE,		/* All files in Tx list are done */

	RF_ERR_FL_IMG_FILE_ERROR,		/* Error in Filename, Image Length */
	RF_ERR_FL_IMG_PAGE_ERROR,		/* Error in page number, Image Length */

	RF_ERR_FL_USERINIT_SIZE,		/* User init string too long */
	RF_ERR_FL_USERRESET_SIZE,		/* User reset string too long */
	RF_ERR_FL_NULL_RESET_STR,		/* Try to execute a null Reset string */
	RF_ERR_FL_COVER_NOT_SET,		/* Cover bit needs to be set */

	/*
	** Transmit Proper
	*/
	RF_ERR_TX_NO_MODEM_CAPS,		/* No Modem Tx Caps */
	RF_ERR_TX_DIS_MISSING,			/* Never got a DIS frame during Tx */
	RF_ERR_TX_CFR_MISSING,			/* Never got a CFR during Tx */
	RF_ERR_TX_DCN_FTT_FALLBACK,	/* Cannot fallback for Tx training */
	RF_ERR_TX_DCN_NOOPER,			/* Send DCN, PIP or PIN received */
   RF_ERR_TX_NO_EOP_RESPONSE,		/* No response from a EOP */
	RF_ERR_TX_DCN_RTX_FALLBACK,	/* Bad page transmission, retrain */
	RF_ERR_TX_BUSY,					/* Line busy on dial out */
	RF_ERR_TX_ERROR_DIAL,			/* Error on dial out */
	RF_ERR_TX_GOT_DCN,				/* Got an unexpected DCN during Tx */
	RF_ERR_TX_NOTX,					/* No doc to transmit */
	RF_ERR_TX_STARVE,					/* Starved the Class 1 */
	RF_ERR_TX_MAKE_DCS,				/* Cannot make a DCS (speed bits) */

	/*
	** Receive FIle name, CSI frame, etc.
	*/
	RF_ERR_RP_PATH_SIZE,				/* Receive path too big */
	RF_ERR_RP_SPEED_RANGE,			/* User asked for Rx out of range speed */
	RF_ERR_RP_SPEED_MODEM,			/* User asked for Rx invalid modem speed */
	RF_ERR_RP_SPEED_DIS,	 			/* User asked for Rx invalid API speed */
	RF_ERR_RP_MEDIA,					/* User specified unknown media */
	RF_ERR_RP_FILE_EXIST,			/* File already exist */
	RF_ERR_RP_INV_POLL_MODE, 		/* Unrecongized poll option */

	/*
	** Receive Proper
	*/
	RF_ERR_RX_ERROR,					/* General error received from modem */
	RF_ERR_RX_DCN_PHASEB_TIMEOUT,	/* No response to our DIS after 38 sec */
	RF_ERR_RX_POST_DCS_TIMEOUT,	/* Post RX Phase B timeout */
	RF_ERR_RX_NO_MODEM_CAPS,		/* No Modem Rx Caps */
	RF_ERR_RX_GOT_DCN,				/* Got an unexpected DCN during Rx */
	RF_ERR_RX_NO_DTC,					/* Rx DTC not accepted */
	RF_ERR_RX_NO_CIGDTC,	 			/* Rx DTC not accepted because of CIG */
	RF_ERR_RX_GOT_PRI,				/* PRI-XXX frame received during Rx */
	RF_ERR_RX_SYS_ABORT,				/* SYS Abort */
	RF_ERR_RX_READ_DCS,				/* Unexpected DCS speed bits set */

	/*
	** Others
	*/
	RF_ERR_GEN_WRONG_TOKEN,			/* Unexpected modem token */
	RF_ERR_GEN_FCERROR,				/* Modem reports wrong modulation */
	RF_ERR_GEN_NO_RESPONSE,			/* No reponse from the modem */
	RF_ERR_GEN_NOCARR,				/* No carrier detect */
	RF_ERR_GEN_NO_FCLASS,			/* Cannot do class 1 */
	RF_ERR_GEN_SYS_ABORT,			/* User abort before data transfer */
	RF_ERR_GEN_FTH3_WATCHDOG,		/* Timeout, xmit hdlc frame mode */
	RF_ERR_GEN_FTM_WATCHDOG,		/* Timeout, TX high speed mode */
	RF_ERR_GEN_FRM_WATCHDOG,		/* Timeout, RX high speed mode */

	/*
	** Internal Errors
	*/
	RF_INV_LAST_XMIT,					/* Inval last frame */
	RF_INV_NEXTFILE,					/* Inval next file */
	RF_INV_POLLMODE,					/* Unrecongized poll option */

	/*
	** T4 Errors
	*/
   RF_ERR_T4_OPEN_FILE_ERROR,   	/* Error found when opening a file */
	RF_ERR_T4_RX_NO_CARRIER1,	  	/* Rx #1: loss of carrier & no initial EOL */
	RF_ERR_T4_RX_NO_INITIAL_EOL, 	/* Rx #2: initial EOL not seen in 10 sec. */
	RF_ERR_T4_RX_EOL_TIMEOUT,	  	/* Rx #3: inter-EOL period > 10 sec. */
	RF_ERR_T4_RX_NO_CARRIER2,	  	/* Rx #4: loss of carrier & no RTC */
	RF_ERR_T4_RX_BAD_LINES,			/* Rx #5: line error count > threshold */
	RF_ERR_T4_RX_OUTPUT_TIMEOUT,	/* Rx #6: Output buffer timeout occurred */
	RF_ERR_T4_ILLEGAL_STATE,		/* #7: illegal state specified */
	RF_ERR_T4_WRITE_FILE,			/* #8: error in writing file */
   RF_ERR_T4_FILE_TYPE_ERROR,   	/* Unrecognized file type */
   RF_ERR_T4_TX_FILE_NO_EOL,	  	/* No EOL seend */

	/*
	** Developer's Error Codes
	*/
	RF_ERR_DEV_INV_IRQ_BUF=-28671,	/*	Developer's use - Irq buffer */
	RF_ERR_DEV_INV_COM_ID,				/* Invalid COM id, 0..(MAX_COM_ID-1) */

	/*
	** Add on errors
	*/
	RF_SPARE=-24575					/* Start of spare errors */
	};

// /*
// ** Maximum Telephone Number accepted
// */
// #define TELE_BUF_SIZE 60
// 
// /*
// ** Maximum Tx File Path
// */
// #define FILE_PATH_SIZE 100
// 
// /*
// ** TSI/CSI buffer size
// */
// #define ID_BUF_SIZE 21
// 
// /*
// ** This API assumes the AT command size buffer on the modem is no
// ** greater than AT_BUF_SIZE
// */
// #define AT_BUF_SIZE 30
// 
// /*
// ** Additional User Init String; do precede with AT
// */
// #define USERINIT_BUF_SIZE 8
// 
// /*
// ** Tx files come from which media (slower Tx speed for floppies)
// */
// #define USE_MEDIA_HD 0
// #define USE_MEDIA_FLOPPY 1
// 
// #define USE_MEDIA_NUM (USE_MEDIA_FLOPPY + 1)
// 
/*
** TSI/CSI buffer size
*/
#define ID_BUF_SIZE 21

/*
** Cover sheet or not
*/
#define USE_COVER_NOSHEET 0
#define USE_COVER_SHEET 1

#define USE_COVER_NUM (USE_COVER_SHEET + 1)

/*
** Merge file or not
*/
#define USE_MERGE_NO 0
#define USE_MERGE_YES 1

#define USE_MERGE_NUM (USE_COVER_SHEET + 1)

/*
** Tx file is what - FAX (TIFF format) or a ASCII file.
*/
#define FORMAT_FAX 0
#define FORMAT_ASCII 1
#define FORMAT_BUFFER 2

#define FORMAT_NUM (FORMAT_BUFFER+1)

/*
** Resolution overrides
*/
#define USE_RES_ASIS 0
#define USE_RES_STANDARD 1
#define USE_RES_FINE 2

#define USE_RES_NUM (USE_RES_FINE + 1)

/*
** Possible T30 Speeds
*/
enum T30SpeedsEnum
	{
	SPEED_2400,
	SPEED_4800,
	SPEED_7200,
	SPEED_7200_V17,
	SPEED_9600,
	SPEED_9600_V17,
	SPEED_12000_V33,
	SPEED_12000_V17,
	SPEED_14400_V33,
	SPEED_14400_V17
	};

#define T30_SPEEDS_NUM (SPEED_14400_V17+1)

/*
** Let T30 determine best speed based on the media and the results
** returned from GetCap().
*/
#define T30_SPEEDS_BEST 15

/*
** Define polling options
*/
enum pollops
	{
	POLL_NONE,
	POLL_TX_UNSECURED,
	POLL_TX_SECURED,
	POLL_RX
	};

#define POLL_NUM (POLL_RX+1)


/*
** Default ASCII page formatting settings.  Margins and Lengths are
** in the units, inches x 100; 1100 is 11 inches.
*/
#define Def_wOpt_Wrap 0
#define Def_wOpt_PageLength 1100 
#define Def_wOpt_Tab 8 
#define Def_wOpt_LeftMargin 45
#define Def_wOpt_RightMargin 45
#define Def_wOpt_TopMargin 50
#define Def_wOpt_BottomMargin 50 
#define Def_wOpt_PageFill 0 

/*
** WatchDog Timer Limits
*/
enum wdde
	{
	WD_INIT,
	WD_GENERAL,
	WD_RING_WAIT,
	WD_FTS,
	WD_FRM,
   WD_RING_COUNT,
   WD_DUMMY
	};

#define WD_NUM (WD_DUMMY+1)

/*
** Line Error Control structure
*/
struct RxLineErrStruct
	{
	unsigned int lthres;
	unsigned int max_n;			/* maximum no. of sequential line errors */
	unsigned int cd_lthres;
	unsigned int cd_max_n;
	};

/*
** Public flags required for adding a file entry.
*/
typedef struct PublicFlagsDef
	{
	unsigned speed 		: 4;		/* Transmit Speed (SPEED_xxxx) */
	unsigned cover 		: 1;		/* Cover Page file */
	unsigned merge			: 1;		/* Merge two files, no EOP */
	unsigned media 		: 1;		/* Media - Hard Disk or not */
	unsigned ResSelect	: 2;		/* Select resoultion */
	unsigned FileFormat	: 2;		/* File type (0=FAX, 1=ASCII, 2=Buffer) */
	unsigned done			: 1;		/* completed file	*/
	} PUBLIC_FLAGS;

/*
** ASCII page format parameters
*/
typedef struct PageFormatDef
	{
   unsigned int wOpt_Wrap;
   unsigned int wOpt_PageLength;
   unsigned int wOpt_Tab;
   unsigned int wOpt_LeftMargin;
   unsigned int wOpt_RightMargin;
   unsigned int wOpt_TopMargin;
   unsigned int wOpt_BottomMargin;
	} PAGE_FORMAT;

/*
** =========================================================================
**
**	T30 FRAME EQUATES
**
** T30 FRAME SIGNAL FORMAT  -  see table in Appendix II of CCITT T30
**
** =========================================================================
*/

#define FrameCFR 0x84
#define FrameDCN 0xFA
#define FrameDCS 0x82
#define FrameDIS 0x80
#define FrameEOM 0x8E
#define FrameEOP 0x2E
#define FrameFTT 0x44
#define FrameMCF 0x8C
#define FrameMPS 0x4E
#define FramePIN 0x2C
#define FramePIP 0xAC
#define FramePRI_EOM 0x9E
#define FramePRI_EOP 0x3E
#define FramePRI_MPS 0x5E
#define FrameRTN 0x4C
#define FrameRTP 0xCC
#define FrameNSF 0x20
#define FrameCSI 0x40
#define FrameTSI 0x42
#define FrameDTC 0x81
#define FrameCIG 0x41

/*
** =======================================================================
**
** Useful macros
**
** =======================================================================
*/
#define cnvtbit(bit) (1 << bit)
#define setbit(word,bit) (word|= cnvtbit(bit))
#define tstbit(word,bit) (word & cnvtbit(bit))
#define clrbit(word,bit) (word&= ~cnvtbit(bit))

/*
** =======================================================================
**
** TIFF File Open Parameters (used in APIT4FF.C)
**
** Should match corresponding values in FAXFILE.H
**
** =======================================================================
*/
#define RFF_READONLY 1
#define RFF_UPDATE 2
#define RFF_CREATE 3

/*
** =======================================================================
**
** Function Prototypes for Interfacing between API and Developer's
** Application
**
** =======================================================================
*/
extern void FAR PASCAL
	RFClose(void);
extern int FAR PASCAL
	RFGetImageLength(
		unsigned char far *filename,int page,
			unsigned int far *length);
extern unsigned char far * FAR PASCAL
	RFGetRemoteIdStr(void);
extern unsigned int FAR PASCAL
	RFGetTxFallBack(void);
extern unsigned int FAR PASCAL
	RFGetTxFBThresh(unsigned int fb);
extern int FAR PASCAL
	RFGetTxStat(int media,int far *Best,int far *Worst);
extern int FAR PASCAL
	RFInit(void);
extern int FAR PASCAL
	RFinProgress(void);
extern void FAR PASCAL
	RFLineErrorControl(
		int set,int FineMode,
			struct RxLineErrStruct far *p);
extern int FAR PASCAL
	RFOpenforCap(void);
extern int FAR PASCAL
	RFOpenforReset(unsigned char far *ResetStr);
extern int FAR PASCAL
	RFOpenforRx(
		int Manual,int Polling,unsigned char far *fn,
			int ReqRxSpeed,int ReqMedia, int Rings);
extern int FAR PASCAL
	RFOpenforTx(int Manual);
extern void FAR PASCAL
	RFPoll(void);
extern void * FAR PASCAL
	RFRetContextPtr(void);
extern unsigned int FAR PASCAL
	RFRetContextSize(void);
extern int FAR PASCAL
	RFSetDialString(unsigned char far *TeleNumber);
extern int FAR PASCAL
	RFSetLocalIdStr(unsigned char far *IdBuffer);
extern int FAR PASCAL
	RFSetUserInitString(unsigned char far *str);
extern int FAR PASCAL
	RFStatus(void);
extern void FAR PASCAL
	RFSysAbortReq(void);
extern void FAR PASCAL
	RFUpdateTimers(
		unsigned int duration);
extern unsigned char far * FAR PASCAL
	RFVersion(void);
extern void FAR PASCAL
	RFWDControl(int set,unsigned int far *wd);

/*
** =======================================================================
**
** Function Prototypes for File List Routines
**
** =======================================================================
*/
extern int FAR PASCAL
	RFAddTxEntry(
		int far *entry, PUBLIC_FLAGS far *fl_flags, 
	   	PAGE_FORMAT far *pagefor, unsigned int wOpt_PageFill,
	      	unsigned int size, void far *filename);
extern int FAR PASCAL
	RFClearTxDones(void);
extern void FAR PASCAL
	RFClearTxEntries(void);
extern int FAR PASCAL
	RFEntriesTxLeft(void);
extern unsigned char far * FAR PASCAL
	RFGetCurRxFile(void);
extern unsigned int FAR PASCAL
	RFGetCurRxPage(void);
extern unsigned int FAR PASCAL
	RFGetCurRxSpeed(void);
extern unsigned char far * FAR PASCAL
	RFGetCurTxFile(void);
extern unsigned int FAR PASCAL
	RFGetCurTxPage(void);
extern unsigned int FAR PASCAL
	RFGetCurTxResolution(void);
extern unsigned int FAR PASCAL
	RFGetCurTxSpeed(void);
extern int FAR PASCAL
	RFGetTxEntry(
		int entry, PUBLIC_FLAGS far *fl_flags, 
	   	PAGE_FORMAT far *pagefor, unsigned int far *wOpt_PageFill,
				unsigned int far *PagesSent,
		      	long far *size, unsigned char far *filename);
extern int FAR PASCAL
	RFMaxTxEntries(void);
extern void far * FAR PASCAL
	RFRetFileList(int far *count);

/*
** =======================================================================
**
** Function Prototypes provided by the Application Developer
**
** =======================================================================
*/
extern void	FAR PASCAL
	RFEndTransReport(
		int code,int line,unsigned char far *fn,
			int userabort);
extern void FAR PASCAL
	RFPollSysTimer(void);
extern void FAR PASCAL
	RFPollSysTimerInit(void);
extern void FAR PASCAL
	RFWarnReport(int code,int line,unsigned char far *fn);
/*
** =======================================================================
**
** Debug Function Prototypes
**
** =======================================================================
*/
extern int FAR PASCAL RFCheckPollPassword(void);
extern void FAR PASCAL
	RFDumpHDLCFrame(
		unsigned char far *ptr,
			int FrameSize,int known,int Rx);
extern unsigned int FAR PASCAL
	RFMaskDISBit(unsigned char FCFByte);
extern void FAR PASCAL
	RFSetTimerTest(unsigned int duration);
extern void FAR PASCAL
	RFShowRxAT(unsigned char far *s);
extern void FAR PASCAL
	RFShowTxAT(unsigned char far *s);
extern unsigned int FAR PASCAL
	RFTimerExpiredTest(void);
extern void FAR PASCAL
	RFDCSSetTest(void);


/*
** =======================================================================
**
** Session Prototypes
**
** =======================================================================
*/
extern int FAR PASCAL
	RFOpenSess(unsigned int id,long param);
extern int FAR PASCAL
	RFCloseSess(void);

/*
** =======================================================================
**
** T4/File Prototypes
**
** =======================================================================
*/
extern int near PASCAL
	RFFWriteBlock(unsigned char *Bufptr,unsigned int number);
extern int near PASCAL RFFWriteDone(void);
extern int near PASCAL RFFClose(void);
extern void near PASCAL RFFDefaultDir(void);
extern void near PASCAL RFFSetResolution(int fine);
extern int near PASCAL RFFInsertPage(void);
extern int near PASCAL RFFOpen(unsigned char far *fn,int mode);
extern int near PASCAL RFFReadDir(int PageNum);
extern unsigned int near PASCAL
	RFFReadBlock(unsigned char *Bufptr,unsigned int number);
extern int near PASCAL RFFGetFineMode(void);
extern int near PASCAL RFFGetLastPage(void);
extern unsigned int near PASCAL RFFGetRawImageLength(void);

