/*
*************************************************************************
**                ROCKWELL CONFIDENTIAL AND PROPRIETARY
**          This source is the sole property of ROCKWELL Inc.
**      Reproduction or utilization of this source in whole or in 
**    part is forbidden without the written consent of ROCKWELL Inc.
*************************************************************************
**                  (c) Copyright ROCKWELL, INC. 1990
**                          All Rights Reserved
*************************************************************************
**
** Last update:   4-26-91
**	Last PVCS update: $Date:   12 Aug 1992 20:06:12  $ 
**
*/


/************************************************************************
*************************************************************************
**
**
**	TITLE:		FAXFILE.H
**
**	FACILITY:	(application, tsr)
**
**
**	Description:	This file contains the definitions and
**			structures used with the FaxFile library.
**
**	Creation Date:	Sept. 20, 1989
**
**
**	Modification History:
**
**		$Log:   D:/api_lib/faxfile.h_v  $
   
      Rev 1.0   12 Aug 1992 20:06:12   Robert Chen
   Initial revision.
** 
**    Rev 1.13   29 Apr 1992 16:29:52   JIM
** Global update of T.30/T.4 API (FAPI) source for version 1.03.
** 
**    Rev 1.10   17 Oct 1991 10:38:32   JIM
** Global update of T.30/T.4 API source for version 1.00.
** 
**    Rev 1.9   12 Sep 1991 14:13:36   JIM
** Global update of T.30/T.4 API source for version 0.22B.
** 
**    Rev 1.7   13 Aug 1991 17:25:44   JIM
** Correction of API source check in to include new fax file source.
** 
**    Rev 1.0   11 Apr 1991 15:09:22   STAN
** Initial revision.
** 
**    Rev 1.6   22 Mar 1991 12:50:22   PHIL
** Updated temporary buffer handling and FF_ReOpen
** 
**    Rev 1.5   12 Feb 1991 10:25:20   PHIL
** Add APIDOS support
** 
**    Rev 1.4   23 Oct 1990 12:08:24   PHIL
** Beta fixes
** 
**    Rev 1.3   02 Aug 1990 10:53:50   PHIL
** Beta Release
** 
**    Rev 1.2   01 May 1990 14:26:38   PHIL
** Windows DLL support
** 
**    Rev 1.1   05 Feb 1990 14:19:16   PHIL
** TSR Version and Merge Function - Concatenate
** 
**    Rev 1.0   27 Nov 1989 14:53:12   PHIL
** Initial revision.
**
**		Test Package - Released Oct. 25, 1989
**
**		Prototype Package - Released Sept. 25, 1989
**
*************************************************************************
************************************************************************/
/****************************************************************
**
**	There are three basic versions for the faxfile
**	package;  Normal "C", "TSR" special, and window's
**	"DLL".
**
**
**	"FF_DLL" controls the conditional compile for windows DLL:
**		The file routines in this version are the standard
**		window's "OpenFile" with the "_lwrite" and "_lread".
**		In a windows DLL all exported functions need to be
**		a "far pascal" type function.
**
**	"FF_TSR" controls the conditional compile for a special TSR.
**		This will exclude the read/write row functions
**		along with the compress/decompress functions.  The
**		TSR will access the Fax File using the read/write
**		block functions.  This is done to save space in the
**		TSR.
**
**	"FF_API" controls the conditional compile for a set special
**		file access routines, which do not use the C library
**		functions.  The special file access functions call
**		the DOS interrupt 21h directly.  This file access
**		functions are located in the "APIDOS.asm" module.
**		The assemble language APIDOS module does not support
**		all memory models, and should only be used for
**		"small" and "medium".
**
*****************************************************************/

/*	The window's "DLL" version	*/
#if	defined	(FF_DLL)
	#define FF_WIN_OF
	#define	DLL	far
	#define	DLLFUNC	far pascal
	#define	ERRNO	-1
	#define	FFDOSWRITE	_lwrite
	#define	FFDOSREAD	_lread
	#define	FFDOSCLOSE	_lclose
	#define	FFDOSSEEK	_llseek
	#ifndef	OF_READ
		/* Disable everthing but "openfile"	*/
		/* why do this, you ask?....		*/
		/*	I want to compile with "W3".	*/
		/*	W3 requires NOMINMAX and NOCOMM.*/
		/*	The rest is needed if we get	*/
		/*	included in an "RC" compile.	*/
		#define NOMINMAX
		#define NOCOMM
		#define NOSCROLL
		#define NOSHOWWINDOW
		#define NOREGION
		#define NOVIRTUALKEYCODES
		#define NOWH
		#define NOPOINT
		#define NOPEN
		#define NORASTEROPS
		#define NOGDI
		#define NOMETAFILE
		#define NOBITMAP
		#define NORECT
		#define NOWNDCLASS
		#define NOMSG
		#define NOWINOFFSETS
		#define NOWINMESSAGES
		#define NONCMESSAGES
		#define NOKEYSTATE
		#define NOWINSTYLES
		#define NOMETAFILE
		#define NOCLIPBOARD
		#define NOHDC
		#define NOCREATESTRUCT
		#define NOTEXTMETRIC
		#define NOGDICAPMASKS
		#define NODRAWFRAME
		#define NODRAWTEXT
		#define NOCTLMGR
		#define NOCLIPBOARD
		#define NOSYSMETRICS
		#define NOMENUS
		#define NOMB
		#define NOCOLOR
		#define NOFONT
		#define NOMEMMGR
		#define NORESOURCE
		#define NOATOM
		#define NOSYSCOMMANDS
		#define NOICON
		#define NOKANJI
		#define NOCTLMGR
		#define NOSOUND

		/* include the windows definitions for "openfile" and #undef those
    ** those symbols defined in windows.h 
    */
    #undef FAR
    #undef PASCAL
    #undef NULL
		#include <windows.h>

	/* and some undocumented window functions */
	WORD	far pascal	_lread  (int, char far *, int) ;
	WORD	far pascal	_lwrite (int, char far *, int) ;
	int	far pascal	_lclose (int) ;
	long	far pascal	_llseek (int, long, int) ;
	#endif
/*	NOT "FF_DLL", normal "C" version	*/
#else
	/*	General items	*/
	#ifndef	FALSE
	#define FALSE 0
	#endif

	#ifndef TRUE
	#define TRUE 1
	#endif


	/*	Type Definitions	*/
	typedef	unsigned char	BYTE ;
	typedef	unsigned short	WORD ;
	typedef	unsigned long	DWORD ;

	/*	NON-DLL mode definitions	*/
	/*	use the C memory model		*/
	#define	DLL	
	#define	DLLFUNC	

	/*	set the file accessing		*/
	#if	defined (FF_API)
		#define	ERRNO		RZerrnum
		#define	FFDOSWRITE	RZDOSWrite
		#define	FFDOSREAD	RZDOSRead
		#define	FFDOSCLOSE	RZDOSClose
		#define	FFDOSSEEK	RZDOSSeek
	#else
		#define	ERRNO		errno
		#define	FFDOSWRITE	FF_DOSWrite
		#define	FFDOSREAD	FF_DOSRead
		#define	FFDOSCLOSE	close
		#define	FFDOSSEEK	lseek
		#define	FF_DEF_DOSREAD
		#define	FF_DEF_DOSWRITE
	#endif
#endif


/*	"BYTE pointers" (DLL version is "far")	*/
typedef	BYTE DLL *FFPBYTE ;



/*	Configuration control - "TSR" version	*/

#if	defined (FF_TSR)
	#define	FF_DEF_TSR
	#define	FF_DEF_READBLK
	#define	FF_DEF_WRITEBLK
#else
	#define	FF_DEF_COPYPAGE
	#define	FF_DEF_MERGE
	#define	FF_DEF_READROW
	#define	FF_DEF_READBLK
	#define	FF_DEF_WRITEROW
	#define	FF_DEF_WRITEBLK
	#define	FF_DEF_FINDROW
	#define FF_DEF_COMPRESS
	#define FF_DEF_DECOMPRESS
	#define FF_DEF_PAGENUMBERTAG
#endif
/*	Error Returns	*/

#define	FF_NOBUFFER		-10
#define	FF_BUFFERTOOSMALL	-11
#define	FF_INVALIDPAR		-12
#define	FF_BYTECOUNT		-13
#define	FF_NOSUCHTAG		-14
#define	FF_NOSUCHPAGE		-15
#define	FF_INVALIDFILE		-16
#define	FF_CONVERTERROR		-17
#define	FF_INVALIDCODE		-18
#define	FF_NOEOL		-19
#define	FF_RUNTOOLONG		-20
#define	FF_INVALIDWIDTH		-21
#define	FF_STRIPTOOSMALL	-22
#define	FF_INVALIDFAXFILE	-23
#define	FF_INVALIDHANDLE	-24
#define FF_IMAGETOOLONG   -25
#define FF_TAGFORMAT      -26
#define FF_TAGDATATYPE    -27

/*	Open Parameters	*/

#define	FF_READONLY		1
#define	FF_UPDATE		2
#define	FF_CREATE		3


/*	Resolution values	*/

#define	FF_STANDARD		100
#define	FF_FINE			200


/*	Compression values	*/

#define	FF_COMPRESS		3
#define	FF_NOCOMPRESS		1


/*	Standard FAX Width	*/

#define	FF_WIDTH		1728


/*	Merge File Parameters	*/

#define	FF_CONCATENATE		1
/****************************************************************
*
*
*	"THE" FaxFile Structure
*
*
*****************************************************************/


/*	The Tag Value definition - needed for FaxFile Structure */

struct	FaxFileTagValueShort
	{
	unsigned int	wValue ;
	unsigned int	wValueH ;
	} ;

union	FaxFileTagValue
	{
	unsigned long	dwValue ;
	struct		FaxFileTagValueShort	Short ;
	} ;


/*	The Import Parameters Structure - needed for FaxFile Structure	*/

struct	ImportParam
	{
	/*	Top-Left starting point, Offset in TIFF image	*/
	WORD	XPos ;		/*	*/
	WORD	YPos ;		/*	*/
	/*	Size of image (portion of TIFF image)	*/
	WORD	XSize ;		/*	*/
	WORD	YSize ;		/*	*/
	} ;


/*	The User's Interface - The FaxFile Structure */

#define	MAXCHR	50
#define	FILE_PATH_SIZE	100

typedef	struct	FaxFile
	{
	BYTE		szFileName[FILE_PATH_SIZE] ;	/* file name in asciz */
	#if	defined (FF_WIN_OF)
	OFSTRUCT	OF ;			/* open-file structure */
	#endif
	BYTE far	*pUserBuffer ;		/* pointer to user's buffer */
	WORD		wUserBufferSize ;	/* size of user buffer (bytes) */
	WORD		wUserByteCount ;	/* size of user data (bytes) */
	WORD		wDebug ;		/* debug control mask */
	DWORD		dwNewSubfileType ;	/* value for tag */
	WORD		wImageWidth ;		/* value for tag */
	WORD		wImageLength ;		/* value for tag */
	WORD		wRowsPerStrip ;		/* value for tag */
	DWORD		dwStripOffset ;		/* value for tag */
	DWORD		dwStripByteCount ;	/* value for tag */
	DWORD		dwXResolution ;		/* value for tag */
	DWORD		dwYResolution ;		/* value for tag */
	WORD		wResolutionUnit ;	/* value for tag */
	WORD		wSamplesPerPixel ;	/* value for tag */
	WORD		wBitsPerSample ;	/* value for tag */
	WORD		wFillOrder ;		/* value for tag */
	WORD		wPhotometricInterpretation ;	/* value for tag */
	WORD		wCompression ;		/* value for tag */
	DWORD		dwGroup3Options ;	/* value for tag */
	WORD		wPageNumber ;		/* value for tag */
	WORD		wPages ;		/* value for tag */
	struct ImportParam ImPar ;		/* import parameters */
	BYTE		szPageName[MAXCHR] ;	/* value for tag */
	BYTE		szImageDescription[MAXCHR] ;	/* value for tag */
	DWORD		dwApplicationStatus ;	/* pointer for tag */
	DWORD		dwTxLog ;		/* pointer for tag */


	/* This area is for exclusive use by the FaxFile library */

	WORD		hFile ;			/* file handle */

	BYTE far	*pTmpBuffer ;		/* pointer to tmp buffer */
	WORD		wTmpBufferCurrent ;	/* current data pointer */
	WORD		wTmpBufferEnd ;		/* end of data pointer */
	WORD		wTmpBufferLastEOL ;	/* last EOL in buffer */
	WORD		wTmpBufferSize ;	/* size of tmp buffer (bytes) */
	WORD		wTmpByteCount ;		/* size of tmp data (bytes) */
	DWORD		fpTmpBuffer ;		/* file offset of tmp buffer */

	int		iUsePageNumbers ;	/* do we use the page number tags */
	WORD		wPageTotal ;		/* total number of pages in file */
  int     bBitRev ;       /* Should bit reversal occur */  

			/* file header */
	WORD		wByteOrder ;		/* byte order in file */
	WORD		wVersion ;		/* TIFF version */
	DWORD		fpFirstDirectory ;	/* file offset (pointer) */

			/* directory */
	int		iUnknownTags ;		/* counter, unknown tags */
	int		iTags ;			/* number of tags in directory */
	DWORD		fpPriorDirectoryEnd ;	/* pointer to prior directory link list */
	DWORD		fpCurrentDirectory ;	/* pointer to current (tag count) */
	DWORD		fpCurrentDirectoryEnd ;	/* pointer to end of current (next pointer) */
	DWORD		fpNextDirectory ;	/* file offset (pointer) */

	WORD		wRowBC ;		/* row byte count (not compressed) */
	DWORD		fpRow ;			/* current row pointer */
	WORD		wRow ;			/* current row number */
	DWORD		fpRow1 ;		/* last row pointer */
	WORD		wRow1 ;			/* last row number */
	DWORD		fpRow2 ;		/* second last row pointer */
	WORD		wRow2 ;			/* second last row number */

			/* current file offset (pointers) for each tag */
	DWORD		fpNewSubfileType ;	/* file offset for tag */
	DWORD		fpImageWidth ;		/* file offset for tag */
	DWORD		fpImageLength ;		/* file offset for tag */
	DWORD		fpRowsPerStrip ;	/* file offset for tag */
	DWORD		fpStripOffset ;		/* file offset for tag */
	DWORD		fpStripByteCount ;	/* file offset for tag */
	DWORD		fpXResolution ;		/* file offset for tag */
	DWORD		fpXResolutionData ;	/* file offset for data */
	DWORD		fpYResolution ;		/* file offset for tag */
	DWORD		fpYResolutionData ;	/* file offset for data */
	DWORD		fpResolutionUnit ;	/* file offset for tag */
	DWORD		fpSamplesPerPixel ;	/* file offset for tag */
	DWORD		fpBitsPerSample ;	/* file offset for tag */
	DWORD		fpFillOrder ;		/* file offset for tag */
	DWORD		fpPhotometricInterpretation ;	/* file offset for tag */
	DWORD		fpCompression ;		/* file offset for tag */
	DWORD		fpGroup3Options ;	/* file offset for tag */
	DWORD		fpPageNumber ;		/* file offset for tag */
	DWORD		fpPageName ;		/* file offset for tag */
	DWORD		fpImPar ;		/* file offset for tag */
	DWORD		fpImParData ;		/* file offset for data */
	DWORD		fpImageDescription ;	/* file offset for tag */
	DWORD		fpApplicationStatus ;	/* file offset for tag */
	DWORD		fpTxLog ;		/* file offset for tag */

			/* Temporary TAG buffer */
	WORD		wTagID ;		/* the tag ID */
	WORD		wTagType ;		/* the tag data type */
	DWORD		dwTagLength ;		/* data length (count) */
	union		FaxFileTagValue	TagValue ;	/* the tag data value */
						/* can be accessed by: */
			/* Normal "long"  = FaxFile.TagValue.dwValue */
			/* "short" (low)  = FaxFile.TagValue.Short.wValue */
			/* "short" (high) = FaxFile.TagValue.Short.wValueH */

	} FAXFILE; /* end of structure */

typedef	FAXFILE DLL *PFAXFILE;
typedef	FAXFILE far *LPFAXFILE;
/****************************************************************
*
*	TIFF File - Definitions
*
*****************************************************************/

#define	FF_BYTEORDER	0x4949			/* Byte Order "II" */
#define	FF_VERSION	0x002A			/* TIFF Version */

/*	TAG ID's */
#define	FF_NEWSUBFILETYPE		254
#define	FF_SUBFILETYPE			255
#define	FF_IMAGEWIDTH			256
#define	FF_IMAGELENGTH			257
#define	FF_BITSPERSAMPLE		258
#define	FF_COMPRESSION			259
#define	FF_PHOTOMETRICINTERPRETATION	262
#define	FF_THRESHHOLDING		263
#define	FF_CELLWIDTH			264
#define	FF_CELLLENGTH			265
#define	FF_FILLORDER			266
#define	FF_DOCUMENTNAME			269
#define	FF_IMAGEDESCRIPTION		270
#define	FF_MAKE				271
#define	FF_MODEL			272
#define	FF_STRIPOFFSETS			273
#define	FF_ORIENTATION			274
#define	FF_SAMPLESPERPIXEL		277
#define	FF_ROWSPERSTRIP			278
#define	FF_STRIPBYTECOUNTS		279
#define	FF_MINSAMPLEVALUE		280
#define	FF_MAXSAMPLEVALUE		281
#define	FF_XRESOLUTION			282
#define	FF_YRESOLUTION			283
#define	FF_PLANARCONFIGURATION		284
#define	FF_PAGENAME			285
#define	FF_XPOSITION			286
#define	FF_YPOSITION			287
#define	FF_FREEOFFSETS			288
#define	FF_FREEBYTECOUNTS		289
#define	FF_GRAYRESPONSEUNIT		290
#define	FF_GRAYRESPONSECURVE		291
#define	FF_GROUP3OPTIONS		292
#define	FF_GROUP4OPTIONS		293
#define	FF_RESOLUTIONUNIT		296
#define	FF_PAGENUMBER			297
#define	FF_COLORRESPONSEUNIT		300
#define	FF_COLORRESPONSECURVES		301
#define	FF_SOFTWARE			305
#define	FF_DATETIME			306
#define	FF_ARTIST			315
#define	FF_HOSTCOMPUTER			316
#define	FF_PREDICTOR			317
#define	FF_WHITEPOINT			318
#define	FF_PRIMARYCHROMATICITIES	319
#define	FF_COLORMAP			320

/*	Private Tags, Allocated by "TIFF Coordinator" at MicroSoft	*/
#define	FF_RI1				33658
#define	FF_RI2				33659
#define	FF_RI3				33660
#define	FF_RI4				33661
#define	FF_RI5				33662

#define	FF_IMPAR			FF_RI1

/*	Tag Data Types	*/
#define	FF_TAGTYPE_BYTE			1
#define	FF_TAGTYPE_ASCII		2
#define	FF_TAGTYPE_SHORT		3
#define	FF_TAGTYPE_LONG			4
#define	FF_TAGTYPE_RATIONAL		5

/*	TIFF Field sizes	*/
#define	FF_TAGSIZE			12
#define	FF_HEADERSIZE			8
#define	FF_SHORTSIZE			2
#define	FF_LONGSIZE			4
/****************************************************************
*
*	FaxFile Function Prototypes
*
*****************************************************************/


/*	(FF_FILE.c) File Level Functions	*/

int DLLFUNC	FF_Open		(PFAXFILE, int) ;
int DLLFUNC	FF_ReOpen	(PFAXFILE, int, int) ;
int DLLFUNC	FF_Close	(PFAXFILE) ;
#if	defined (FF_DEF_MERGE)
int DLLFUNC	FF_Merge	(PFAXFILE, PFAXFILE, int) ;
#endif


/*	(FF_DIR.c) Directory Level Functions	*/

int DLLFUNC	FF_ReadDir	(PFAXFILE, int) ;
int DLLFUNC	FF_ReadNextDir	(PFAXFILE) ;
void DLLFUNC	FF_DefaultDir	(PFAXFILE) ;
int DLLFUNC	FF_CopyPage	(PFAXFILE, int, PFAXFILE, int) ;
int DLLFUNC	FF_InsertPage	(PFAXFILE, int) ;
int DLLFUNC	FF_FindDir	(PFAXFILE, int) ;
int DLLFUNC	FF_DeletePage	(PFAXFILE, int) ;
int DLLFUNC	FF_WriteDir	(PFAXFILE) ;
int DLLFUNC	FF_AdjustPages	(PFAXFILE) ;


/*	(FF_TAG.c) Tag Level Functions	*/

int DLLFUNC	FF_WriteDone	(PFAXFILE) ;
int DLLFUNC	FF_ReadTag	(PFAXFILE, int, int) ;
int DLLFUNC	FF_ReadNextTag	(PFAXFILE) ;
int DLLFUNC	FF_ProcessTag	(PFAXFILE) ;
int DLLFUNC	FF_WriteTag	(PFAXFILE) ;


/*	(FF_IMAGE.c) Image Level Functions	*/

#if	defined (FF_DEF_READROW)
int DLLFUNC	FF_ReadRow	(PFAXFILE, WORD, int) ;
#endif

#if	defined (FF_DEF_READBLK)
int DLLFUNC	FF_ReadBlock	(PFAXFILE) ;
#endif

#if	defined (FF_DEF_FINDROW)
int DLLFUNC	FF_FindRow	(PFAXFILE, WORD) ;
#endif

#if	defined (FF_DEF_WRITEROW)
int DLLFUNC	FF_WriteRow	(PFAXFILE, int) ;
#endif

#if	defined (FF_DEF_WRITEBLK)
int DLLFUNC	FF_WriteBlock	(PFAXFILE) ;
#endif
/*	(COMRESS.c) Date compression Function	*/

#if	defined (FF_DEF_COMPRESS)
int DLLFUNC	FF_Compress	(BYTE far *, WORD, BYTE far *, int, int DLL *) ;
int DLLFUNC	FF_CompressRuns	(int far *, WORD, BYTE far *, int, int DLL *) ;
#endif


/*	(DECOMPR.asm) Date decompression Function	*/

#if	defined (FF_DEF_DECOMPRESS)
int far FF_Decompress (BYTE far *, WORD, BYTE far *, WORD, WORD far *, WORD far *) ;
#endif


/*	(FF_STRUC.c) FaxFile Structure Functions	*/

void DLLFUNC FF_Set_Standard (PFAXFILE) ;
void DLLFUNC FF_Set_Fine	(PFAXFILE) ;
void DLLFUNC FF_Set_Compress (PFAXFILE) ;
void DLLFUNC FF_Set_NoCompress (PFAXFILE) ;


/*	(FF_RW.c) General File Access Functions	*/

int DLLFUNC	FF_Seek		(PFAXFILE, DWORD) ;
int DLLFUNC	FF_WriteShort	(PFAXFILE, int) ;
int DLLFUNC	FF_WriteLong	(PFAXFILE, long) ;
int DLLFUNC	FF_WriteBuffer	(PFAXFILE, BYTE far *, WORD) ;
int DLLFUNC	FF_ReadShort	(PFAXFILE, FFPBYTE) ;
int DLLFUNC	FF_ReadLong	(PFAXFILE, FFPBYTE) ;
int DLLFUNC	FF_ReadBuffer	(PFAXFILE, BYTE far *, WORD) ;


/*	DOS Level - File Access Functions	*/

int DLLFUNC	FF_DOSRead	(WORD, BYTE far *, WORD) ;
int DLLFUNC	FF_DOSWrite	(WORD, BYTE far *, WORD) ;


/*	(FF_ERR.c) Standard FaxFile Error Message	*/

#if	defined (FF_DLL)
void DLLFUNC	FF_ErrorMessage (HWND, int, LPSTR, LPSTR) ;
int DLLFUNC FF_GetErrorStr( int, LPSTR, WORD );
#endif
