/*
***************************************************************************
**              ROCKWELL CONFIDENTIAL AND PROPRIETARY
**        This source is the sole property of ROCKWELL Inc.
**    Reproduction or utilization of this source in whole or in 
**  part is forbidden without the written consent of ROCKWELL Inc.
***************************************************************************
**                 (c) Copyright ROCKWELL, INC. 1991
**                       All Rights Reserved
***************************************************************************
*/

/* 
** Last update:   
**	Last PVCS update: $Date:   12 Aug 1992 19:58:36  $ 
*/

/*
**	TITLE:		rawdlg4.c
**
**	FACILITY:
**
**	MODULE DESCRIPTION:
**
*/

/*
**
**	Revision History:
**
**		$Log:   D:/api_lib/rawdlg4.c_v  $
 * 
 *    Rev 1.0   12 Aug 1992 19:58:36   Robert Chen
 * Initial revision.
** 
**    Rev 1.5   17 Oct 1991 10:40:34   JIM
** Global update of T.30/T.4 API source for version 1.00.
** 
**    Rev 1.4   12 Sep 1991 11:17:56   JIM
** Global update of T.30/T.4 API source for version 0.22B.
** 
**    Rev 1.3   08 Aug 1991 18:47:00   JIM
** Update to facilitate single session DLL version of API.
** 
**    Rev 1.0   13 May 1991 09:58:22   JIM
** Initial Revision
** 
** 
*/
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "zapi.h"
#include "fapi.h"
#include "rawglobl.h"
#include "rawclas1.h"
#include "raw.h"

static BOOL NEAR IsWild( char * );
static VOID NEAR SelectFile( HWND );

char szPropertyName [] = "FILENAME";/* Name of the File name property list item */

//---------------------------------------------------------------------------
//  FUNCTION   : IsWild ( psz )
//								
//  PURPOSE    : Checks if the string (referenced by a NEAR pointer)
//		 contains a DOS wildcard character ("*" or "?").	 
//								
//  RETURNS    : TRUE  - iff the string contains a wildcard character. 
//		 FALSE - otherwise.				  
//---------------------------------------------------------------------------
static BOOL NEAR IsWild( register PSTR psz )
{
  for ( ; ; )
    switch ( *psz++ )
    {
      case '*' :
      case '?' :

        // Found wildcard 
        return TRUE;

      case 0   :

        // Reached end of string 
        return FALSE;

      default  :
        continue;
	  }
}

//---------------------------------------------------------------------------
//  FUNCTION   : SelectFile ( hwnd )	
// 
//  PURPOSE    : Reads the string in the edit control of the File/Open
//		 dialog. If it contains a wildcard, then it attempts to     
//		 fill the listboxes in the File/Open dialog. Othewise it    
//		 ends the dialog. Modifies the FILENAME item in the property
//		 list of the window. 
//---------------------------------------------------------------------------
static VOID NEAR SelectFile( register HWND hwnd )
{
  register PSTR pch;
  PSTR	  pch2;

  // Get handle (near address) to filename data in window's property list 
  pch = ( PSTR ) GetProp( hwnd, PROP_FILENAME );

  // Get the text from the dialog's edit control into this address
  GetDlgItemText( hwnd, ED_FILENAME, pch, 64 );

  if ( IsWild( pch ) )
  {
    // Select the directory and make a listing of the directories 
    DlgDirList(hwnd, (LPSTR)pch, LD_DIRS, SD_PATH, ATTR_DIRS);

    // Obtain the filename-only part of the path in the edit control 
    for ( pch2 = pch; *pch; pch++ )
      if ( *pch == '\\' || *pch == ':' )
        pch2 = pch + 1;

    // List the files in this directory based on the wildcard. 
    DlgDirList( hwnd, ( LPSTR )pch2, LD_FILES, SD_PATH, ATTR_FILES );

    // Set the dialog's edit control to the filename part of path
    // string.
    SetDlgItemText( hwnd, ED_FILENAME, pch2 );
  }
  else
  {
    // The filename in the property list is not a wildcard 
    if ( FileExists( hwnd, ( LPSTR ) pch, "Open File...", ED_FILENAME ) )
    {
      RemoveProp( hwnd, PROP_FILENAME );
      EndDialog( hwnd, IDOK );
    }
    else
      SetActiveWindow( hwnd );
  }
}

#pragma optimize("egl",off)     // Temporarily deactivates global optimizations.
//---------------------------------------------------------------------------
//  FUNCTION   : OpenFileWndProc()					    
//
//  PURPOSE    : Dialog function for the File/Open dialog. Takes care of    
//		 calling the appropriate functions for extracting the	    
//		 filename and wildcard, filling the listboxes and changing  
//		 the FILENAME item in the property list for the window.     
//---------------------------------------------------------------------------
BOOL FAR PASCAL OpenFileWndProc( register HWND hwnd, WORD message, 
                                 WORD wParam, LONG lParam )
{
  PSTR pch;

  switch (message)
  {
    case WM_INITDIALOG:

      // Set the default file extension on edit window, and try to
      // get a listing of the files and directories.
      SetDlgItemText( hwnd, ED_FILENAME, szDefFileSrch );
      SetProp( hwnd, PROP_FILENAME, LOWORD( lParam ) );
      SetWindowText( hwnd, ( LPSTR ) ( NPSTR ) HIWORD( lParam ) );
      SendDlgItemMessage( hwnd, ED_FILENAME, EM_LIMITTEXT, 64, 0L );
      SelectFile( hwnd );
      break;

    case WM_COMMAND:
      switch (wParam)
      {
        case IDOK:
          SelectFile(hwnd);
          break;

        case IDCANCEL:
          // Set the filename in the prop. list to NULL and quit 
          pch  = ( PSTR ) GetProp( hwnd, PROP_FILENAME );
          *pch = 0;
          EndDialog (hwnd, 0);
          break;

        case ED_FILENAME:
          // Enable the OK button if the edit control has text. 
          EnableWindow ( GetDlgItem (hwnd, IDOK),
		      GetWindowTextLength ((HWND)LOWORD (lParam)));
          break;

        case LD_FILES:

          // The files listbox. If file selection has changed, fill
          // the new filename into the property list buffer and set
          // text in edit control.
          if (HIWORD(lParam) == LBN_SELCHANGE)
          {
		         pch = ( PSTR ) GetProp( hwnd, PROP_FILENAME );
		         DlgDirSelect( hwnd, (LPSTR)pch, LD_FILES );
		         SetDlgItemText( hwnd, ED_FILENAME, ( LPSTR )pch );
          }
          else 
            if ( HIWORD( lParam ) == LBN_DBLCLK )
		           // if the item was double-clicked, try to open it 
		           SelectFile(hwnd);
          break;

        case LD_DIRS:

          // The directories listbox. Append current filename in edit
          // control (stripped of the path prefix) to the name from
          // the property list and set the new string in the edit
          // control.

          if ( HIWORD(lParam) == LBN_SELCHANGE )
          {
            PSTR pch2, pchT, pchS;

            pch = ( PSTR ) GetProp( hwnd, PROP_FILENAME );

            // Get the new drive/dir 
            DlgDirSelect( hwnd, pch, LD_DIRS );
            pch2 = pch + lstrlen( pch );

            // Fetch current contents of dialog's edit control and append
            // it to name from property list... 
            GetDlgItemText( hwnd, ED_FILENAME, ( LPSTR ) pch2, 64 );

            if ( *pch2 == 0 )
            {
              SetDlgItemText( hwnd, ED_FILENAME, szDefFileSrch );
              GetDlgItemText( hwnd, ED_FILENAME, ( LPSTR )pch2, 64 );
            }
            else 
            {
              pchS = pch;
              for ( pchT = pch = pch2; *pch; pch++ ) 
              {
                if ( *pch == '\\' || *pch == ':' )
                  pchT = pch2;
                else
                  *pchT++ = *pch;
              }
              *pchT = 0;
              pch = pchS;
            }
            // Set the edit control with new string 
            SetDlgItemText( hwnd, ED_FILENAME, ( LPSTR )pch );
          }
          else 
            if ( HIWORD( lParam ) == LBN_DBLCLK )
              SelectFile( hwnd );
          break;

        default:
          return FALSE;
      }
      break;

    default:
      return FALSE;
  }
  return TRUE;
}
#pragma optimize("",on)  // Restores global optimization settings from CL.             

//---------------------------------------------------------------------------
//  FUNCTION   : GetFilename ( pstr )		    
//								    
//  PURPOSE    : Gets a filename from the user by calling the File/Open
//		     dialog.	 
//---------------------------------------------------------------------------
int GetFileName(PSTR pstr, PSTR szCaption)
{
    FARPROC lpfn;
    int iReturn;

    lpfn = MakeProcInstance (OpenFileWndProc, hInst);
    iReturn = DialogBoxParam (hInst, "OpenFile", hWndMain, lpfn, 
                              MAKELONG(pstr,szCaption));
    FreeProcInstance (lpfn);

    return ( iReturn );
}

//---------------------------------------------------------------------------
//  FUNCTION   : SaveAsDlgProc(hwnd, message, wParam, lParam)
//									
//  PURPOSE    : Dialog function File/SaveAs. It waits for a filename, and
//    	     then calls SaveFile or cancels the operation.
//---------------------------------------------------------------------------
BOOL FAR PASCAL SaveAsWndProc( HWND hwnd, WORD message, WORD wParam, LONG lParam )
{
  static char *szDefFileName;
  static char szWindowTitle[ 40 ];
  char szTmpFileName[ _MAX_PATH ];

  switch (message)
  {
    case WM_INITDIALOG:
      szDefFileName = ( NPSTR ) LOWORD( lParam );

      // Set window caption to describe selected menu option.
      SetWindowText( hwnd, ( LPSTR ) ( NPSTR ) HIWORD( lParam ) );
      strcpy( szWindowTitle, ( NPSTR ) HIWORD( lParam ) );

    	/* Display the filename in the edit control */
    	SetDlgItemText( hwnd, ED_SAVETO, szDefFileName );

    	/* Select the entire range of text */
      SendDlgItemMessage( hwnd, ED_SAVETO, EM_SETSEL, 0, MAKELONG( 0, 100 ) );

      DlgDirList( hwnd, "*.*", LD_DIRS, SD_PATH, ATTR_DIRS );

      /* enable OK button iff edit control is nonempty */
      if ( !*szDefFileName )
	      EnableWindow( GetDlgItem( hwnd, IDOK ), FALSE );
      break;

    case WM_COMMAND:
      switch ( wParam )
      {
        case IDCANCEL:
          // Abort operation 
          EndDialog( hwnd, IDCANCEL );
          break;

        case IDOK:
          GetDlgItemText( hwnd, ED_SAVETO, szTmpFileName, _MAX_PATH );

          if ( FileValidate( hwnd, szTmpFileName, szWindowTitle, ED_SAVETO ) )
            strcpy( szDefFileName, szTmpFileName );
          else
            break;

          EndDialog( hwnd, IDOK );
          break;

        case ED_SAVETO:
          // If the edit control changes, check to see if its empty.
          // enable OK if it contains something
          //
          if ( HIWORD( lParam ) != EN_CHANGE )
    	      return ( FALSE );
          EnableWindow( GetDlgItem( hwnd, IDOK ),
    	                  ( WORD ) SendDlgItemMessage( hwnd, ED_SAVETO,
       				          WM_GETTEXTLENGTH, 0, 0L ) );
          break;

        case LD_DIRS:
          if ( HIWORD( lParam ) == LBN_DBLCLK )
          {
    	      char szT[ 64 ];

    	      DlgDirSelect( hwnd, szT, LD_DIRS );
    	      lstrcat( szT, "*.*" );
    	      DlgDirList( hwnd, szT, LD_DIRS, SD_PATH, ATTR_DIRS );
    	      break;
          }
          return FALSE;

        default:
          return FALSE;
      }

    default:
      return FALSE;
  }
  return TRUE;
}

int SaveAsFileName( NPSTR szFileName, NPSTR szCaption )
{
  FARPROC lpfnSaveAsWndProc;
  int iReturn;

  lpfnSaveAsWndProc = MakeProcInstance( SaveAsWndProc, hInst );
  iReturn = DialogBoxParam( hInst, "SaveFile", hWndMain, lpfnSaveAsWndProc, 
                            MAKELONG( ( WORD ) szFileName, ( WORD ) szCaption ) );
  FreeProcInstance( lpfnSaveAsWndProc );
  return( iReturn );
}

