/*
***************************************************************************
**              ROCKWELL CONFIDENTIAL AND PROPRIETARY
**        This source is the sole property of ROCKWELL Inc.
**    Reproduction or utilization of this source in whole or in 
**  part is forbidden without the written consent of ROCKWELL Inc.
***************************************************************************
**                 (c) Copyright ROCKWELL, INC. 1991
**                       All Rights Reserved
***************************************************************************
*/

/* 
** Last update:   
**	Last PVCS update: $Date:   12 Aug 1992 19:58:44  $ 
*/

/*
**	TITLE:		rawdos.c
**
**	FACILITY:
**   File I/O Developer Functions
**
**	MODULE DESCRIPTION:
**   The functions is this module perform basic disk I/O and are called by
**   the API.  Their calling syntax and return values are simlar to the
**   C standard runtime library functions for file I/O.
*/

/*
**
**	Revision History:
**
**		$Log:   D:/api_lib/rawdos.c_v  $
 * 
 *    Rev 1.0   12 Aug 1992 19:58:44   Robert Chen
 * Initial revision.
** 
**    Rev 1.2   17 Oct 1991 10:40:42   JIM
** Global update of T.30/T.4 API source for version 1.00.
** 
**    Rev 1.1   12 Sep 1991 11:18:02   JIM
** Global update of T.30/T.4 API source for version 0.22B.
** 
**    Rev 1.0   08 Aug 1991 18:47:06   JIM
** Update to facilitate single session DLL version of API.
** 
**    Rev 1.0   13 May 1991 09:56:04   JIM
** Initial Revision
** 
** 
*/
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "rawglobl.h"
#include "rawerr.h"
#include "rawsdio.h"
#include "zapi.h"
#include "zapidos.h"
#include "zapicls1.h"

char szTmpStr[ TMPSTRMAX ];

//---------------------------------------------------------------------------
// The value of this global error variable is accessible by using the 
// access function GetAPIFileErr.  The fax file functions which previously
// used the value of this variable would have to be appropriately modified
// if their status as a DLL were to change. JYW
//---------------------------------------------------------------------------
int RZerrnum;

//---------------------------------------------------------------------------
// Function RFDosOpen - Calls the Windows function OpenFile to the open the
//                      DOS file as directed.
// Author:
//   James Y. Wilson
// Description:
//   This function opens the file specified using the method specified. 
//   The windows function OpenFile is used to perform the actual open.
//   The contents of szPath is changed to reflect the fully qualified path
//   name of the file originally referenced by this argument.
// Args:
//   szPath - Points to the file name and later the fully qualified path name
//     of the file opened.  The character string referenced by this
//     parameter should be defined using the constant _MAX_PATH in
//     windows.h to allow the longer path name to replace the file
//     name originally referenced in this argument.
//   iMethod - The access method to be used when opening the file.
// Returns:
//   A non negative value indicates the file handle to be used in the
//   call to other DOS file I/O functions.  A negative value indicates
//   the opening of the file failed.
//---------------------------------------------------------------------------
int FAR PASCAL RZDOSOpen( unsigned char far *szPath, int iMethod )
{
  int iReturn;
  int iWinMethod;
  OFSTRUCT ofStruct;

  if ( !szPath[ 0 ] )
    return ( RZerrnum = -1 );   // ERROR: Invalid DOS path name in parm.

  switch ( iMethod )
  {
    case ORF_RDONLY:
      iWinMethod = OF_READ;
      break;
    case ORF_WRONLY:
      iWinMethod = OF_WRITE;
      break;
    case ORF_RDWR:
      iWinMethod = OF_READWRITE;
      break;
    default:
      return ( RZerrnum = -1 ); // ERROR: Invalid access code requested.
  }

  iReturn = OpenFile( szPath, &ofStruct, iMethod );

//  if ( iReturn == -1 )
//    wputs( ( LPSTR ) StrError( "RZDOSOpen", IS_CANTOPEN, szPath, ofStruct.nErrCode ) );

  RZerrnum = ( iReturn == -1 ? -1 : 0 );

  return ( iReturn ); // Return error code provided by
}                                          // Windows file I/O routine.

//---------------------------------------------------------------------------
// Function:  RZDOSCreate - This function creates the specified file.
//
// Author:
//   James Y. Wilson
// Description:
//   This function calls the Windows OpenFile function using create
//   method of open and replaces the file name referenced in the szPath
//   argument with the fully qualified path name.
// Args:
//   szPath - Points to the file name and later the fully qualified path name
//     of the file created.  The character string referenced by this
//     parameter should be defined using the constant _MAX_PATH in
//     windows.h to allow the longer path name to replace the file
//     name originally referenced in this argument.
// Return:
//   A non negative value indicates the file handle to be used in calls to
//   other DOS file I/O functions.  Any other file indicates failure in
//   the attempt to create the specified file.
//---------------------------------------------------------------------------
int FAR PASCAL RZDOSCreate( unsigned char far *szPath )
{
  int iReturn;
  OFSTRUCT ofStruct;

  if ( !szPath[ 0 ] )
    return ( RZerrnum = -1 );               // ERROR: Invalid DOS path name in parm.

  iReturn = OpenFile( szPath, &ofStruct, OF_CREATE );

//  if ( iReturn == -1 )
//    wputs( GetFocus(), ( LPSTR ) StrError( GetFocus(), "RZDOSCreate:", IS_CANTCREATE, 
//                                           szPath, ofStruct.nErrCode ) );
  RZerrnum = ( iReturn == -1 ? -1 : 0 );

  return ( iReturn );  // Return error code provided by Windows
}                                   // file I/O routine.

// Return Value: -1 is an error, 0 is success.
int FAR PASCAL RZDOSDelete( unsigned char far *szPath )
{
  lstrcpy( ( LPSTR ) szTmpStr, szPath );
  return ( RZerrnum = remove( ( NPSTR ) szTmpStr ) );
}

//---------------------------------------------------------------------------
// Function:  RZDOSClose - Close the file specified by the handle.
//
// Author:
//   James Y. Wilson
// Description:
//   The file referenced by the iHandle parameter is closed.  Notice
//   the update of the global variable RZerrnum. 
// Args:
//   iHandle - The handle used to reference the file previously opened or
//     create using the RFOpenFile or RFCreateFile functions.
// Return:
//   A return value of 0 indicates success.  A -1 indicates failure.
//---------------------------------------------------------------------------
int FAR PASCAL RZDOSClose( int iHandle )
{
  return ( RZerrnum = _lclose( iHandle ) ); // Return error code provided by 
}                                    // Windows file I/O routine.

//---------------------------------------------------------------------------
// Function:  RZDOSRead - Reads a certain number of bytes from a previously
//                        opened or created file.
// Author:
//   James Y. Wilson
// Description:
//   This function reads the number of bytes specified in the argument iCount
//   into the buffer pointed to by the argrument sBuffer.  The argument
//   iHandle references a previously opened or created file.
// Args:
//   iHandle - The DOS file handle assigned in a call to RFOpenFile or
//     RFCreateFile.
//   sBuffer - Points to the area of memory where the characters read
//     would be placed.
//   iCount - The number of characters to read into the area of memory
//     pointed to by the sBuffer argument.
// Return:
//   A non negative value indicates the number of bytes successfully read.
//   A negative value indicates that an error occured when reading the file.
//                        
// Return Value: -1 is an error, otherwise return value equals number 
// of bytes successfully read.
//---------------------------------------------------------------------------
int FAR PASCAL RZDOSRead( int iHandle, unsigned char far *sBuffer, unsigned int iCount )
{
  int iReturn;

  iReturn = _lread( iHandle, sBuffer, iCount );
  if ( iReturn == -1 )
    WRError( GetFocus( ), "RZDOSRead", IS_CANTREAD, iHandle );
  RZerrnum = ( iReturn == -1 ? -1 : 0 );
  return ( iReturn );
}

//---------------------------------------------------------------------------
// Function:  RZDOSWrite - Writes the a buffer of a certain length to a DOS
//                         file.
// Author:
//   James Y. Wilson
// Description:
//   This function calls the Windows file I/O function for writing to the
//   file specified by the argument iHandle.  The API calls this function
//   for writing to a DOS file.
// Args:
//   iHandle - The DOS file handle assigned in a call to RFOpenFile or
//     RFCreateFile.
//   sBuffer - Points to the area of memory where the bytes to be
//     written are located.
//   iCount - The number of characters in the argument sBuffer to write 
//     to the file referenced in iHandle.
// Return:
//   A non negative value indicates the number of bytes successfully
//   written.  A negative value indicates a failure in writing to the
//   specified DOS file.  
//---------------------------------------------------------------------------
int FAR PASCAL RZDOSWrite( int iHandle, unsigned char far *sBuffer, unsigned int iCount )
{
  int iReturn;

  iReturn = _lwrite( iHandle, sBuffer, iCount );
  if ( iReturn == -1 )
    WRError( GetFocus( ), "RZDOSWrite", IS_CANTWRITE, iHandle );
  RZerrnum = ( iReturn == -1 ? -1 : 0 );

  return ( iReturn );
}

//---------------------------------------------------------------------------
// Function:  RZDOSSeek - Positions the file pointer to a certain offset
//                        relative to a certain starting point.
// Author:
//   James Y. Wilson
// Description:
//   This function repositions the file pointer to the location specified
//   using the method specified.  This function is called by the API
//   to advance to a particular byte within a binary file.
// Args:
//   iHandle - The DOS file handle assigned in a call to RFOpenFile or
//     RFCreateFile.
//   lFOffset - The number of bytes to move from the location described
//     by the iMethod argument.
//   iMethod - The method and starting location when positioning to the 
//     byte specified in the argument lFOffset.
// Return:
//   A non negative value represents the absolute offset from the
//   beginning of the file.  This may or may not correspond to the
//   offset specified in the argument lFOffset, depending on the method
//   chosen for relocating the file pointer.  A negative value 
//   indicates an error in performing the file seek.
//---------------------------------------------------------------------------
long FAR PASCAL RZDOSSeek( int iHandle, long lFOffset, int iMethod )
{
  long lReturn;

  lReturn = _llseek( iHandle, lFOffset, iMethod );

  RZerrnum = ( lReturn == -1 ? -1 : 0 );
  return ( lReturn );  // Return error code provided
}                      // by Windows file I/O routine.

