/*
***************************************************************************
**              ROCKWELL CONFIDENTIAL AND PROPRIETARY
**        This source is the sole property of ROCKWELL Inc.
**    Reproduction or utilization of this source in whole or in 
**  part is forbidden without the written consent of ROCKWELL Inc.
***************************************************************************
**                 (c) Copyright ROCKWELL, INC. 1991
**                       All Rights Reserved
***************************************************************************
*/

/* 
** Last update:   
**	Last PVCS update: $Date:   07 Sep 1992 15:45:32  $ 
*/

/*
**	TITLE:		rawmain.c
**
**	FACILITY:
**   Windows application level functions.
**
**	MODULE DESCRIPTION:
**   The functions in this module demonstrate the integration of the API
**   in an existing end user application as additional menu options to
**   to initiate transmit/receive transactions.  The View and Print modules
**   are activated as stand alone modules currently not included in the 
**   API deliverable.
*/

/*
**
**	Revision History:
**
**		$Log:   D:/api_lib/rawmain.c_v  $
 * 
 *    Rev 1.3   07 Sep 1992 15:45:32   Robert Chen
 *  
 * 
 *    Rev 1.2   18 Aug 1992 16:42:36   Robert Chen
 * 
 *    Rev 1.1   16 Aug 1992 22:49:08   Robert Chen
 * 
 *    Rev 1.0   12 Aug 1992 19:59:18   Robert Chen
 * Initial revision.
** 
**    Rev 1.6   17 Oct 1991 10:41:08   JIM
** Global update of T.30/T.4 API source for version 1.00.
** 
**    Rev 1.5   12 Sep 1991 11:18:34   JIM
** Global update of T.30/T.4 API source for version 0.22B.
** 
**    Rev 1.4   08 Aug 1991 18:47:38   JIM
** Update to facilitate single session DLL version of API.
** 
**    Rev 1.1   20 May 1991 09:14:18   JIM
** 
**    Rev 1.0   13 May 1991 10:02:50   JIM
** Initial Revision
** 
** 
*/

//---------------------------------------------------------------------------
// Command Line Constants:
// WIN_TEST -  Maximum level of progress messages.  This may not allow 
//             normal operation of the test module due to excessive delays 
//             between output statments.
//---------------------------------------------------------------------------
#include <windows.h> 
#include <stdlib.h>  
#include <stdio.h>   
#include <string.h>  
#include "zapi.h"     
#include "fapi.h"
#include "vapi.h"
#include "vcallbk.h"

#include "rawglobl.h"
#include "rawclas1.h"
#include "rawsdio.h" 
#include "rawerr.h"
#include "view.h"    
#include "print.h"   
#include "raw.h"	    
#include "rawvoice.h"
#include "example.h"

// Declarations of application global variables.
char     szXLInitFileName[ _MAX_PATH ]; // Default Xmit List configuration file.
char     szCPInitFileName[ _MAX_PATH ]; // Default Conf Parm configuration file.
FARPROC  lpfnOldWndProc; // Pointer to Main Window proc prior to subclassing

// Local functions declarations:
static BOOL NEAR InitApplication( HANDLE );
static BOOL NEAR InitInstance( HANDLE, int );
static VOID NEAR CommandHandler( HWND, WORD, LONG );

//---------------------------------------------------------------------------
// Function:  WinMain - Main level function called by Windows.
//
// Author:
//   James Y. Wilson
// Description:
//   This function performs all Windows message dispatches.  When the
//   PeekMessage function has determined that no other application messages
//   remain to be processed an IU_FAXPOLL message is posted which causes the
//   the fax polling loop function (RFFaxPoll) to be called.
// Args:
//   Defined in Windows SDK documentation.
// Return:
//   Defined in Windows SDK documentation.
//---------------------------------------------------------------------------
int PASCAL WinMain( HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpCmdLine, 
                    int nCmdShow )
{
  MSG msg;

  if ( !hPrevInstance )   
    if ( !InitApplication( hInstance ) )  // Initialize shared objects 
      return ( FALSE );	                  // Exits if unable to initialize 

  if ( !InitInstance( hInstance, nCmdShow ) )
    return ( FALSE );

  while ( TRUE )
  {
    if ( PeekMessage( ( LPMSG ) &msg, NULL, 0, 0, PM_REMOVE ) )
    {
      if ( msg.message == WM_QUIT )
        break;
      if ( !TranslateAccelerator( hWndMain, hAccelMain, &msg ) )
      {
        TranslateMessage( ( LPMSG ) &msg );
        DispatchMessage( ( LPMSG ) &msg );
      }
    }
    else
    {
      // if ( HIWORD( FaxInProgress( ) ) != FAX_REST )
      //   PostMessage( hWndMain, WM_COMMAND, IU_FAXPOLL, 0L );
      // else if ( HIWORD( VoiceInProgress( ) ) != VOICE_REST )
      //   PostMessage( hWndMain, WM_COMMAND, IU_VOICEPOLL, 0L );
#ifdef DO_FAX
      if ( HIWORD( FaxInProgress( ) ) != FAX_REST )
        PostMessage( hWndMain, WM_COMMAND, IU_FAXPOLL, 0L );
#ifdef DO_VOICE
      if ( HIWORD( VoiceInProgress( ) ) != VOICE_REST )
        PostMessage( hWndMain, WM_COMMAND, IU_VOICEPOLL, 0L );
#endif
#endif
    }
  }
  return ( ( int ) msg.wParam );   // Returns the value from PostQuitMessage. 
}

//---------------------------------------------------------------------------
// Function:  InitApplication - Calls functions which must be called only when
//                              first application instance is loaded.
//
// Author:
//   James Y. Wilson
// Description:
//   This function calls the DLL function which registers the window class
//   used to display API progress messages.  It will only be called once
//   when the first application instance is loaded.
// Args:
//   hInstance -
//     Windows handle to the application instance.
//   nCmdShow -
//     Windows parameter indicating the user requested status of the main
//     application window.
// Return:
//   Returns FALSE if application initialization fails, and TRUE if it
//   succeeds.
//---------------------------------------------------------------------------
static BOOL NEAR InitApplication( HANDLE hInstance )
{
  if ( !StdioInit( hInstance ) )
    return ( FALSE );
  return ( TRUE );
}

//---------------------------------------------------------------------------
// Function:  InitInstance - Call functions which must called when each 
//                           application instance is loaded.
//
// Author: 
//   James Y. Wilson
// Description:
//   This function initializes application level global variables, loades
//   configuration data from an .INI file if it exists, creates the main
//   application window, and initializes the local copy of the transmit list
//   maintained as a Windows property list.
// Args:
//   hInstance -
//     Windows handle to the application instance.
//   nCmdShow -
//     Windows parameter indicating the user requested status of the main
//     application window.
// Return:
//   Returns TRUE if the execution of initialization functions was successful
//   and FALSE if one of the functions failed.
//--------------------------------------------------------------------------
static BOOL NEAR InitInstance( HANDLE hInstance, int nCmdShow )
{
  HWND hWnd;

  char szAppName[ TMPSTRMAX ];
  char szIniFileName[ _MAX_PATH ];
  char szIniXLKeyFile[ _MAX_PATH ];
  char szIniCPKeyFile[ _MAX_PATH ];
  char szIniXLDefFile[ _MAX_PATH ];
  char szIniCPDefFile[ _MAX_PATH ];

  // Load parameters to be used in GetPrivateProfileString function.
  LoadString( hInstance, IS_APPNAME, szAppName, sizeof( szAppName ) );
  LoadString( hInstance, IS_INIFILENAME, szIniFileName, sizeof( szIniFileName ) );
  LoadString( hInstance, IS_INIXLKEYFILE, szIniXLKeyFile, sizeof( szIniXLKeyFile ) );
  LoadString( hInstance, IS_INICPKEYFILE, szIniCPKeyFile, sizeof( szIniCPKeyFile ) );
  LoadString( hInstance, IS_INIXLDEFFILE, szIniXLDefFile, sizeof( szIniXLDefFile ) );
  LoadString( hInstance, IS_INICPDEFFILE, szIniCPDefFile, sizeof( szIniCPDefFile ) );

  // Load name of Xmit List and Conf Parm configuation files.
  GetPrivateProfileString( szAppName, szIniXLKeyFile, szIniXLDefFile,
                           szXLInitFileName, sizeof( szXLInitFileName ),
                           szIniFileName );
  GetPrivateProfileString( szAppName, szIniCPKeyFile, szIniCPDefFile,
                           szCPInitFileName, sizeof( szCPInitFileName ),
                           szIniFileName );

  if ( CPInit( szCPInitFileName ) )
    return ( FALSE );

  // Create a main window for this application instance.  
  hWnd = CreateStdioWindow( 
      "Rockwell API  Demo",            // Text for window title bar.
      WS_OVERLAPPEDWINDOW | WS_VSCROLL,// Window style.                       
      CW_USEDEFAULT,                  // Default horizontal position.        
      CW_USEDEFAULT,                  // Default vertical position.          
      CW_USEDEFAULT,                  // Default width.                      
      CW_USEDEFAULT,                  // Default height.                     
      NULL,                           // Overlapped windows have no parent.  
      hInstance,                      // This instance owns this window.     
      TRUE,
      LogEnvDlg.wScrlBuf              // Amount of memory for screen buffer.
  );

  // If window could not be created, return "failure"  
  if ( !hWnd )
    return ( FALSE );
  else
    hWndMain = hWnd;

  // Subclass the main window.
  lpfnOldWndProc = ( FARPROC ) SetWindowLong( hWnd, GWL_WNDPROC, 
                                              ( DWORD ) MainWndProc );
  // Add menu to the main window.
  SetMenu( hWnd, LoadMenu( hInstance, "RawMenu" ) );

  // Set application icon.
  SetClassWord( hWnd, GCW_HICON, LoadIcon( hInstance, "RawIcon" ) );

  // Load accelerator table for main window menu.
  hAccelMain = LoadAccelerators( hInstance, "MainMenuAccel" );
#ifdef DO_FAX
  if ( XLInit( hWnd, szXLInitFileName ) )
    return ( FALSE );
#endif

  // Make the window visible; update its client area; and return "success"  
  ShowWindow( hWnd, nCmdShow );  // Show the window                         
  UpdateWindow( hWnd );          // Sends WM_PAINT message                  
  hInst = hInstance;             // Save handle to application instance.

  return ( TRUE );               // Returns the value from PostQuitMessage  
}

//---------------------------------------------------------------------------
// Function:  MainWndProc - The callback function for the main application
//                          window.
// Author:  
//   James Y. Wilson
// Description:
//   This function is called by Windows to service the messages of the main
//   application window.  It is the result of subclassing of the standard
//   callback function for this window contained in the DLL RAWCLAS1.DLL.
//   If an action is not defined for the message than control is transferred
//   to the old callback function.
// Args:
//   Defined in the Windows SDK documentation.
// Return:
//   Defined in the Windows SDK documentation.
//---------------------------------------------------------------------------
LONG FAR PASCAL MainWndProc( HWND hWnd, WORD wMessage, WORD wParam, LONG lParam )
{
  switch ( wMessage ) 
  {
	  case WM_COMMAND:     // wMessage: command from application menu.
       CommandHandler( hWnd, wParam, lParam );
       break;

	  case WM_DESTROY:		 // wMessage: window being destroyed
#ifdef DO_FAX
       XLDel( hWnd, 0, XLItems( ) ); // Eliminate Xmit List entries in Prop. List.
       FaxClose( );       // Shutdown T.30/T.4 API iterface module.
#endif

       // -------------------------------------------------------------------
       // Fall through to allow Stdio window to perform necessary cleanup.
       // -------------------------------------------------------------------
	  default:			       // passes it on if unproccessed   
	    return ( CallWindowProc( lpfnOldWndProc, hWnd, wMessage, wParam, lParam ) );
  }
  return ( NULL );
}

//---------------------------------------------------------------------------
// Function:  CommandHandler - Process those messages generated due to the
//                             selection of a menu option.
// Author:
//   James Y. Wilson
// Description:
//   This function processes those message received due to end user selection
//   of a defined menu option.  It also processes the message posted for 
//   T30/T4 polling.
// Args:
//   Simply a transfer of those parameters received in the callback function.
// Return:
//   No return value.
//---------------------------------------------------------------------------
static VOID NEAR CommandHandler( HANDLE hWnd, WORD wParam, LONG lParam )
{
  static char szXLFileName[ _MAX_PATH ];
  static char szCPFileName[ _MAX_PATH ];

  switch ( wParam )
  {
    case IM_ABOUT:
      CallDialog( "AboutBox", AboutWndProc, hWnd, 0L );
      break;

    // File menu item selections:
    case IM_EXIT:
      DestroyWindow( hWnd );
      break;

#ifdef DO_FAX
    case IM_FRECEIVE:
      if ( CallDialog( "FileName", FileNameWndProc, hWnd, 
                       ( LONG ) ( LPFILENAME ) &ReceiveDlg ) != IDCANCEL )
        FaxInit( wParam );  // Activate application state machine.
      break;

    case IM_FTRANSMIT:
      // Eventually check return value of this function and display error.
      if ( !*DialStrDlg.szDialStr )
        WRError( hWnd, "Command Handler Error", IS_NODIALSTR );
      else
        FaxInit( wParam );  // Activate application state machine.
      break;

    case IM_FVIEW:
		 {
		   HWND hWndListBox;

		 	 if ( ( hWndListBox = XLOpenLB( hWnd ) ) )
		 	 {
          DoView( hWnd, hWndListBox );
			 	 XLCloseLB( hWndListBox );
		 	 }
		 	 else
		   	 WRError( hWnd, "IM_FVIEW", IS_LISTBOXERR );
		 }
      break;

    case IM_FPRINT:
      {
        HWND hWndListBox;

        if ( ( hWndListBox = XLOpenLB( hWnd ) ) )
    		 {
          DoPrint( hWnd, hWndListBox );
			   XLCloseLB( hWndListBox );
        }
		   else
			   WRError( hWnd, "IM_FPRINT", IS_LISTBOXERR );
      }
      break;

    // ----------------------------------------------------------------------
    // Process active fax transaction.
    // ----------------------------------------------------------------------
    case IU_FAXPOLL:
      // If Multitasking is disabled do not allow control to return to main
      // message loop.
      if ( ConfigFlags.bfMultiTask ) // Has the "Multitasking" menu item been checked?"
      {
        if ( FaxPoll( ) == FAX_ERROR )  // Perform fax transaction polling.
        {
          sprintf( szTmpStr, "Error Code: %i", RFStatus( ) );
          wputs( hWnd, szTmpStr );
        }
      }
      else                // Multitasking is enabled, yield control to windows.
      {
        while ( HIWORD( FaxInProgress( ) ) != FAX_REST )
        {
          if ( FaxPoll( ) == FAX_ERROR )  // Perform fax transaction polling.
          {
            sprintf( szTmpStr, "Error Code: %i", RFStatus( ) );
            wputs( hWnd, szTmpStr );
          }
        }
      }
      break;

    case IM_TVIEW:
      PrintTxList( );
      break;

    case IM_CINITIALIZE:
      FaxInit( wParam );  // Activate application state machine for
      break;              // fax modem initialization only.

    case IM_TOPEN:
      LoadString( hInst, IS_DEFXLFILESRCH, szDefFileSrch, sizeof( szDefFileSrch ) );
      if ( GetFileName( szXLFileName, "Transmit List Open..." ) == IDOK )
        XLLoad( hWnd, szXLFileName );
      break;

    case IM_TEDIT:
      CallDialog( "XmitList", XmitListWndProc, hWnd, 0L );
      break;

#endif
#if defined(DO_FAX)  && defined(DO_VOICE)
    case IM_VTELCOFAX:
//      if ( CallDialog( "FileName", FileNameWndProc, hWnd, 
//                        ( LONG ) ( LPFILENAME ) &VARecordDlg ) != IDCANCEL )
        VoiceInit( IM_VTELCOFAX, FMDlg.wComPort, FMDlg.wRecvTo );
      break;

#endif
    case IM_FCLRSCRN:
      wputs( hWnd, "\f" );
      break;

#ifdef DO_VOICE
    case IM_VHANDMICRO:
      if ( VoiceHandSet( '1', FMDlg.wComPort ) )
		  WRError( hWnd, "IM_HANDMICRO", IS_VOICEPOLL );
      break;

    case IM_VMACHINE1:
      if ( VoiceAnswerMachine( NULL, FMDlg.wComPort ) )
        WRError( hWnd, "IM_VANSWERMACHINE", IS_VOICEPOLL );
      break;

    case IM_VMACHINE2:
      if ( VoiceAnswerMachineCD( NULL, FMDlg.wComPort ) )
        WRError( hWnd, "IM_VANSWERMACHINE", IS_VOICEPOLL );
      break;

#endif

    case IM_TSAVE:
      if ( *szXLFileName )
        XLSave( szXLFileName );
      else
      {
        strcpy( szXLFileName, szXLInitFileName );
        if ( SaveAsFileName( szXLFileName, "Transmit List Save..." ) == IDOK )
          XLSave( szXLFileName );
        else
          *szXLFileName = 0;  // Return to old file name, of null.
      }
      break;

    case IM_TSAVEAS:
      {
        char szOldFileName[ _MAX_PATH ];

        LoadString( hInst, IS_DEFCPFILESRCH, szDefFileSrch, sizeof( szDefFileSrch ) );
        strcpy( szOldFileName, szXLFileName );

        if ( !*szXLFileName )
          strcpy( szXLFileName, szXLInitFileName );

        if ( SaveAsFileName( szXLFileName, "Transmit List Save As..." ) == IDOK )
          XLSave( szXLFileName );
        else                  // Return to old file name
          strcpy( szXLFileName, szOldFileName );
      }  
      break;

    case IM_CLOGGING:
      CallDialog( "LogEnv", LogEnvWndProc, hWnd, 0L );
      break;

    case IM_CDIALSTRING:
      CallDialog( "DialStr", DialStrWndProc, hWnd, 0L );
      break;

    case IM_CSUBID:
      CallDialog( "SubId", SubIdWndProc, hWnd, 0L );
      break;

    case IM_CFAXMODEM:
      CallDialog( "FaxModem", FaxModemWndProc, hWnd, 0L );
      break;

    case IM_CASCIIFORM:
      CallDialog( "AsciForm", AsciFormWndProc, hWnd, 0L );
      break;

    case IM_CMULTITASK:
      ConfigFlags.bfMultiTask = ( BYTE ) ( !ConfigFlags.bfMultiTask );
      CheckMenuItem( GetMenu( hWnd ), IM_CMULTITASK, 
                     ( ConfigFlags.bfMultiTask ? MF_CHECKED : MF_UNCHECKED ) );
      break;

    case IM_CREXMIT:
      ConfigFlags.bfReXmit = ( BYTE ) ( !ConfigFlags.bfReXmit );
      CheckMenuItem( GetMenu( hWnd ), IM_CREXMIT,
                     ( ConfigFlags.bfReXmit ? MF_CHECKED : MF_UNCHECKED ) );
      break;

    case IM_COPEN:
      LoadString( hInst, IS_DEFCPFILESRCH, szDefFileSrch, sizeof( szDefFileSrch ) );
      if ( GetFileName( szCPFileName, "Configuration Open..." ) == IDOK )
        CPLoad( szCPFileName );

      break;

    case IM_CSAVE:
      if ( *szCPFileName )
        CPSave( szCPFileName );
      else
      {
        strcpy( szCPFileName, szCPInitFileName );
        if ( SaveAsFileName( szCPFileName, "Configuration Save..." ) == IDOK )
          CPSave( szCPFileName );
        else
          *szCPFileName = 0;  // Return to old file name, of null.
      }
      break;

    case IM_CSAVEAS:
      {
        char szOldFileName[ _MAX_PATH ];

        strcpy( szOldFileName, szCPFileName );

        if ( !*szCPFileName )
          strcpy( szCPFileName, szCPInitFileName );

        if ( SaveAsFileName( szCPFileName, "Configuration Save As..." ) == IDOK )
          CPSave( szCPFileName );
        else                  // Return to old file name
          strcpy( szCPFileName, szOldFileName );
      }  
      break;

    case IM_CANCEL:
      DisableCancel( hWnd, TRUE );
      break;

#if defined(DO_FAX)  && defined(DO_VOICE)
    // ----------------------------------------------------------------------
    // Process active voice transaction.
    // ----------------------------------------------------------------------
    case IU_VOICEPOLL:
      if ( VoicePoll( ) == VOICE_ERROR )
      {
        sprintf( szTmpStr, "Error Code: %i", RVStatus( ) );
        wputs( hWnd, szTmpStr );
      }
      break;
#endif 

    default:
      WRError( hWnd, "CommandHandler", IS_UNDERCNSTRCT );
      break;
  }
}

  



