/*
***************************************************************************
**              ROCKWELL CONFIDENTIAL AND PROPRIETARY
**        This source is the sole property of ROCKWELL Inc.
**    Reproduction or utilization of this source in whole or in 
**  part is forbidden without the written consent of ROCKWELL Inc.
***************************************************************************
**                 (c) Copyright ROCKWELL, INC. 1991
**                       All Rights Reserved
***************************************************************************
*/

/* 
** Last update:   
**	Last PVCS update: $Date:   12 Aug 1992 19:59:22  $ 
*/

/*
**	TITLE:		rawsdio1.c
**
**	FACILITY:
**
**	MODULE DESCRIPTION:
**
*/

/*
**
**	Revision History:
**
**		$Log:   D:/api_lib/rawsdio1.c_v  $
 * 
 *    Rev 1.0   12 Aug 1992 19:59:22   Robert Chen
 * Initial revision.
** 
**    Rev 1.5   17 Oct 1991 10:41:14   JIM
** Global update of T.30/T.4 API source for version 1.00.
** 
**    Rev 1.4   12 Sep 1991 11:18:38   JIM
** Global update of T.30/T.4 API source for version 0.22B.
** 
**    Rev 1.3   08 Aug 1991 18:47:46   JIM
** Update to facilitate single session DLL version of API.
** 
**    Rev 1.0   13 May 1991 09:57:06   JIM
** Initial Revision
** 
** 
*/
//	FILE:    WSTDIO.c

//	PURPOSE: Contains functions for managing a standard I/O window.
//          Provides a means for easily sending text to a window 
//          for debugging, etc.

//	FUNCTIONS:
//          SetupStdioDC() - Initializes DC.
//          ResetStdioDC() - Selects former font into DC.
//          GetStdioLine() - Returns pointer to specified line in buffer.
//          StdioUpdate() - Scroll and update displayed text.
//          PutStdio() - Process I/O window messages.
//          StdioPaint() - Paint procedure for I/O window.
//          StdioInit() - Define/register I/O window class.
//          wopen() - Create a default I/O window.
//          CreateStdioWindow() - Create a customized I/O window.
//          wputs() - Puts a string in the I/O window.
//          StdioWndProc() - Processes messages for the I/O window.
//                          
// Make all functions near functions in this module near, except wputs, wopen.

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <memory.h>
#include <sys\timeb.h>
#include "zapi.h"
#include "rawglobl.h"
#include "rawclas1.h"
#include "rawsdio.h"
#include "rawsdio.d"
#include "rawerr.h"

//---------------------------------------------------------------------------
// Local Constants:
//---------------------------------------------------------------------------
WORD    BUFSIZE;      
int     LASTBUFPAGE;
int     MAXBUFLINES;

GLOBALHANDLE hScrBuf;
SCRBUF       sScrBuf;	    // Array of characters on TTY.

//---------------------------------------------------------------------------
// Declaration of local global variables:
//---------------------------------------------------------------------------
short  nCurrPos; 		      // Current TTY line output position.
short  nStdioCharWidth,   // width and height of Stdio font chars.
       nStdioCharHeight;
int    iLastBufLine;		  // Index of last line on TTY in the array.
int    iVScrLineCnt;      // Number of lines contained in screen buffer.
int    iVScrollPos;       // Current scrolling position in allowable range.
int    iVScrollMax;       // Upper limit in scrolling range.

DWORD  StdiotextRGB;      // text color
HWND   hWndStdio = NULL;  // handle to standard I/O window 
HANDLE hStdioInst;
BOOL   bStdioQuit;
BOOL   bInited = FALSE;
BOOL   bScrollPainted;    // Indicates that scroll bar is being updated.
BOOL   bScrollEnabled = TRUE;
BOOL   bMinMax;

//--------------------------------------------------------------------------
//
// FUNCTION: ResetStdioDC(HDC)
// PURPOSE:  Prepare to release the DC by selecting the system font.
//
//--------------------------------------------------------------------------
//void ResetStdioDC(HDC hDC)
//{
//  SelectObject( hDC, hOldFont );
//}

//--------------------------------------------------------------------------
//
// FUNCTION: StdioUpdate( HWND, int, BOOL )
// PURPOSE:  Scroll the window by the number of lines we have received,
//           and display the text in the invalidated areas.
//
// ScrollWindow may be called with a scrolling rectangle in parameter 4.  This
// option was elected to allow scrolling of a specific textual area of the
// client space.  This would intern reveal areas of the screen to write new
// text beginning in the precis area associated with the last line of painted
// text.  Since ScrollWindow will not invalidate the appropriate areas of 
// the client space exposed by this scrolling action, the InvalidateRect 
// function is called in the PutStdio function.  
//
// Using a NULL scrolling rectangle would produce faster execution of the
// scrolling action.  A problem arises, however, since the entire client 
// space is scrolled from the edge of the resize frame.  This creates an
// invalid region that is slightly lower than required since text
// is not painted from the very bottom of the window.  Manually generating
// the invalid regions will not resolve this problem, since they will be
// offset during scrolling. ( The "scrolling" of invalid regions occurs when
// parameter 4 is NULL. )  The end result of all this is a series of 
// non-continuous invalid regions all improperly positioned.  A nightmare to
// say the least!
//
// This problem would not exist, if scrolling occurred after the painting
// of each text line.  This would, however, produce "choppy" results on 
// slower systems.
//--------------------------------------------------------------------------- 
VOID _fastcall StdioUpdate( int iLinesOut, BOOL bRect )
{
  RECT rcRect;

  GetClientRect( hWndStdio, &rcRect );

  // Scroll screen by number of lines received.
  if ( bRect )
  {
    AddScrLine( iLinesOut );
    ScrollWindow( hWndStdio, 0, -nStdioCharHeight * iLinesOut, &rcRect, NULL );
  }
  else
  {
    ScrollWindow( hWndStdio, 0, -nStdioCharHeight * iLinesOut, NULL, NULL );
  }
  bScrollPainted = TRUE;   
  SetScrollRange( hWndStdio, SB_VERT, 0, iVScrollMax, FALSE );
  SetScrollPos( hWndStdio, SB_VERT, iVScrollPos, TRUE );
  bScrollPainted = FALSE;   
  UpdateWindow( hWndStdio );
}

#pragma optimize("egl",off)     // Temporarily deactivates global optimizations.
//--------------------------------------------------------------------------
//
// FUNCTION: PutStdio(HWND, HDC, WORD, LPSTR)
// PURPOSE:  Process incoming text to Stdio window.
//
//--------------------------------------------------------------------------
VOID _fastcall PutStdio( HDC hDC, WORD wParam, LPSTR lParam )
{
  short register iNdx;
  short iLinesOut = 0;           // # of lines to scroll
  short iCharCnt = 0;
  LPSTR sBuffer;
  RECT  rcInvalid;

  GetClientRect( hWndStdio, &rcInvalid );

  // If first character of output string is a form feed avoid positioning
  // to bottom since the client area will be completely cleared anyway.
  if ( *lParam == '\f' )
    SendMessage( hWndStdio, WM_VSCROLL, SB_CLEAR, 0L );
  else if ( iVScrollPos < iVScrollMax )
    SendMessage( hWndStdio, WM_VSCROLL, SB_BOTTOM, 0L );

  sBuffer = sScrBuf + iLastBufLine * MAXLINELEN;

  // Scan the text, handle any special characters, and display the rest.
  for ( iNdx = 0; iNdx < ( short ) wParam; ++iNdx )
  {
	  switch ( lParam[ iNdx] )
    {
      case '\r': // return
        if ( lParam[ iNdx+ 1 ] != '\n' )
        {
          // Invalidate the area of the current line.
          rcInvalid.top    = MAXSCRLINE;
          rcInvalid.bottom = rcInvalid.top + 1;
          rcInvalid.left   = nCurrPos - iCharCnt;
          rcInvalid.right  = nCurrPos;
          LPtoDP( hDC, ( LPPOINT ) &rcInvalid, 2 );
          InvalidateRect( hWndStdio, &rcInvalid, TRUE );
          nCurrPos = iCharCnt = 0;   // Reset the current position in the line.
        }
        break;

      case '\n': // new line
        ++iLinesOut;    // Increment lines to scroll.
        ++iLastBufLine; // Increment last line of circular display buffer.

        if ( iLastBufLine > ( int ) MAXBUFLINE )
          iLastBufLine = 0;

        sBuffer = sScrBuf + iLastBufLine * MAXLINELEN;
        _fmemset( sBuffer, '\0', MAXLINELEN - 1 );

        // Invalidate the future area of the current line after it is
        // scrolled.
        rcInvalid.top    = MAXSCRLINE - iLinesOut;
        rcInvalid.bottom = rcInvalid.top + 1;
        rcInvalid.left   = nCurrPos - iCharCnt;
        rcInvalid.right  = nCurrPos;
        LPtoDP( hDC, ( LPPOINT ) &rcInvalid, 2 );
        InvalidateRect( hWndStdio, &rcInvalid, TRUE );
        nCurrPos = iCharCnt = 0;   // Reset the current position in the line.

        break;

      case '\f': // form feed (clear screen)
        // Do not allow clearing on fresh screen.
        if ( !iVScrLineCnt )
          break;

        // The current size of the client area specifies the number of lines
        // to scroll.
        iLinesOut += rcInvalid.bottom / nStdioCharHeight;

        // Determine if buffer must be cleared in two steps.
        if ( iLastBufLine + iLinesOut - 1 > MAXBUFLINE )
        {
          _fmemset( sBuffer, '\0',
                    ( MAXBUFLINE - iLastBufLine + 1 ) * ( MAXLINELEN - 1 ) );
          iLastBufLine = iLinesOut - ( MAXBUFLINE - iLastBufLine - 1 );
          _fmemset( sScrBuf, '\0', ( iLastBufLine + 1 ) * ( MAXLINELEN - 1 ) );
        }
        else
        {
          _fmemset( sBuffer, '\0', iLinesOut * ( MAXLINELEN - 1 ) );
          iLastBufLine += iLinesOut;
        }
        InvalidateRect( hWndStdio, NULL, TRUE );
        nCurrPos = 0;

        break;

      case '\b': // backspace
        // move back one space
        if ( nCurrPos > 0 )
        {
          --nCurrPos;
          sBuffer[ nCurrPos ] = '\0';
        }
        break;

	    case '\t':
        // ignore tabs for now
        break;

      default:
        // add char to buffer
        if ( nCurrPos < 80 )
        {
          // put the character in the screen buffer
          sBuffer[ nCurrPos ] = lParam[ iNdx]; // add char to screen line
          ++nCurrPos;
          ++iCharCnt;
        }
        break;
    }

    // Force scroll after specified number of lines. Scrolling will occur 
    // more frequently, but it may not look good on slower systems.
    if ( iLinesOut > SCROLLINC )
    {
      StdioUpdate( iLinesOut, TRUE );
      iLinesOut = 0;
    }
  }

  // If charcters remain which will not be painted in the client area revealed
  // by scrolling, invalidate the this area now.
  if ( nCurrPos  )
  {
    rcInvalid.top = MAXSCRLINE;
    rcInvalid.bottom = MAXSCRLINE + 1;
    rcInvalid.left = nCurrPos - iCharCnt;
    rcInvalid.right = nCurrPos;
    LPtoDP( hDC, ( LPPOINT ) &rcInvalid, 2 );
    InvalidateRect( hWndStdio, &rcInvalid, TRUE );
  }

  // Force output of remaining characters and lines.
  if ( iLinesOut || nCurrPos )            // Update display if undrawn lines 
    StdioUpdate( iLinesOut, TRUE ); // or characters remain in buffer.
}
#pragma optimize("",on)  // Restores global optimization settings from CL.             

LONG _fastcall AddScrLine( int iLinesOut )
{
  RECT rcRect;

  GetClientRect( hWndStdio, &rcRect );

  // Increment number of lines in screen buffer only if it is currently less
  // than maximum and update scroll bar thumb.
  if ( iVScrLineCnt < MAXBUFLINE )
    iVScrLineCnt = min( iVScrLineCnt + iLinesOut, MAXBUFLINE );

  if ( bScrollEnabled )
    iVScrollMax = iVScrollPos = 
      max( 0, ( bMinMax ? iVScrLineCnt + 1 : iVScrLineCnt )
           - rcRect.bottom / nStdioCharHeight );

  return ( iVScrLineCnt );
}

#pragma optimize("eg",off)     // Temporarily deactivates global optimizations.
//--------------------------------------------------------------------------
// FUNCTION: StdioPaint()
// PURPOSE:  The I/O window paint procedure.  Draws necessary text in 
//           the window.
//--------------------------------------------------------------------------
VOID _fastcall StdioPaint( VOID )   
{
  LPSTR psLine;
  register short iNdx;
  int nVPaintBeg, nVPaintEnd, nHPaintBeg, nHPaintEnd;
  PAINTSTRUCT ps;
  HDC         hDC;
  RECT        rcUpdate;

  hDC = BeginPaint( hWndStdio, ( LPPAINTSTRUCT ) &ps );

  if ( !( sScrBuf = ( SCRBUF ) GlobalLock( hScrBuf ) ) )
  {
    WRError( hWndStdio, "StdioPaint", IS_NOLOCK );
    return;
  }

  // Set viewport extents and mapping mode.
  SetupStdioDC( hDC );

  // Copy the paint coordinates to a local RECT structure and convert to
  // device coordinates.
  rcUpdate = ps.rcPaint;
  DPtoLP( hDC, ( LPPOINT ) &rcUpdate, 2 );

  //-------------------------------------------------------------------------
  // Calculate first and last lines to update. Enlarge paintable area to allow
  // correct repaint of paritally invalidated regions.  The DPtoLP function
  // performs its translation without accounting for roundoff error.
  //-------------------------------------------------------------------------
  if ( ps.rcPaint.top % nStdioCharHeight )
    --rcUpdate.top;

  if ( ps.rcPaint.bottom % nStdioCharHeight )
    ++rcUpdate.bottom;

  nVPaintBeg = max( 0, rcUpdate.top );
  nVPaintEnd = min( MAXSCRLINES, rcUpdate.bottom );

  if ( ps.rcPaint.left % nStdioCharWidth )
    --rcUpdate.left;

  if ( ps.rcPaint.right % nStdioCharWidth )
    ++rcUpdate.right;

  // Calculate the first and last columns to update.
  nHPaintBeg = max( 0, rcUpdate.left );
  nHPaintEnd = min( MAXLINELEN - 1, rcUpdate.right );

  if ( nVPaintBeg == nVPaintEnd )
  {
    psLine = GetStdioLine( nVPaintBeg ) + nHPaintBeg;
    TextOut( hDC, nHPaintBeg, nVPaintBeg, psLine, lstrlen( psLine ) );
  }
  else
  {
    // Display the lines that need to be drawn.
    for( iNdx = nVPaintBeg; iNdx< nVPaintEnd; iNdx++ )
    {
      // Obtain character string to paint and add horizontal offset of paintable
      // region.
      psLine = GetStdioLine( iNdx) + nHPaintBeg;
      TextOut( hDC, nHPaintBeg, iNdx, psLine, lstrlen( psLine ) );
    }
  }

  GlobalUnlock( hScrBuf );
  EndPaint( hWndStdio, ( LPPAINTSTRUCT ) &ps );
}
#pragma optimize("",on)  // Restores global optimization settings from CL.             

//--------------------------------------------------------------------------
//
// FUNCTION: SetupStdioDC( HDC )
// PURPOSE:  Sets up the I/O window DC. Called at GetDC time.
//
// The formula which appears below was designed to allow positioning of
// the first text line of the client space to appear evenly on a
// character cell boundary.  This prevents the clipping of a portion 
// of a line of text at the upper left corner of the client space.  The
// position of the last row of text ( row 25 ) will vary as the window
// is resized.
//
// Position of the origin      ( MAXSCRLINE - rcRect.bottom / nStdioCharHeight )
// mapped to the virtual area 
// immediately above the upper 
// left corner of the client 
// space, in line units:
// 
// Position of the origin in   ( * nStdioCharHeight )
// device units:
//--------------------------------------------------------------------------
VOID _fastcall SetupStdioDC( HDC hDC )
{
  RECT rcRect;
  WORD wOrg;

  GetClientRect( hWndStdio, &rcRect );

  wOrg = -( ( bMinMax ? MAXSCRLINES : MAXSCRLINE ) -
            rcRect.bottom / nStdioCharHeight )
            * nStdioCharHeight;
  
  SetViewportOrg( hDC, 0, wOrg );
    
  SetMapMode( hDC, MM_ANISOTROPIC );

  // Set the extents such that one unit horizontally or 
  // vertically is one character width or height.
  SetWindowExt( hDC, 1, 1 );

  // Set the viewport such that the last line in the buffer is 
  // displayed at the bottom of the window. 
  SetViewportExt( hDC, nStdioCharWidth, nStdioCharHeight );

  SelectObject( hDC, GetStockObject( STDIO_FONT ) );
  SetTextColor( hDC, GetSysColor( COLOR_WINDOWTEXT ) );
  SetBkColor( hDC, GetSysColor( COLOR_WINDOW ) );
  SetBkMode( hDC, OPAQUE );
}

//--------------------------------------------------------------------------
// FUNCTION: StdioStart( )
// PURPOSE:  Initialize variables used by I/O window for each instance.
//--------------------------------------------------------------------------
int StdioStart( HWND hWnd )
{
  HDC hDC;
  TEXTMETRIC Metrics;

  // Get the text metrics for the font we are using.
  hDC = GetDC( hWnd );
  SelectObject( hDC, GetStockObject( STDIO_FONT ) );
  GetTextMetrics( hDC, ( LPTEXTMETRIC ) &Metrics );
  ReleaseDC( hWnd, hDC );

  // Calculate the height and width of the font 
  nStdioCharWidth = Metrics.tmMaxCharWidth;
  nStdioCharHeight = Metrics.tmHeight + Metrics.tmExternalLeading;

  // All CAPS to denote constant value initialized from dialog box variables.
  MAXBUFLINES = BUFSIZE / MAXLINELEN;
  LASTBUFPAGE = MAXBUFLINES - MAXSCRLINES;

  bInited = TRUE;

  return ( 0 );
}

//--------------------------------------------------------------------------
// FUNCTION: StdioInit(HANDLE)
// PURPOSE:  Initialize the stdio module. Registers the window class.
// RETURNS:  Status of RegisterClass().
//--------------------------------------------------------------------------
BOOL FAR PASCAL StdioInit( HANDLE hInstance )
{
  PWNDCLASS pStdioClass;

  if ( hInstance == NULL )
    return ( FALSE );

  // Create the stdio window
  pStdioClass = ( PWNDCLASS ) LocalAlloc( LPTR, sizeof( WNDCLASS ) );

  pStdioClass->hCursor        = LoadCursor( NULL, IDC_ARROW );
  pStdioClass->lpszClassName  = ( LPSTR ) "Stdio";
  pStdioClass->hbrBackground  = COLOR_WINDOW + 1;
  pStdioClass->hInstance      = hInstance;

  // Do not allow closing from the system menu.  Restore bit map if possible.
//  pStdioClass->style		       = CS_SAVEBITS | CS_VREDRAW | CS_OWNDC;
  pStdioClass->style		       = CS_SAVEBITS | CS_VREDRAW | CS_PARENTDC;
  pStdioClass->lpfnWndProc    = StdioWndProc;

  // Allocate extra room to save STDIODATA.
  pStdioClass->cbWndExtra     = STDIOWINEXTRA;

  if ( !RegisterClass( ( LPWNDCLASS ) pStdioClass ) )
    return ( FALSE );

  LocalFree( (HANDLE) pStdioClass );

  return ( TRUE );
}

//--------------------------------------------------------------------------
// FUNCTION: wopen(HWND, BOOL)
// PURPOSE:  Create a default style stdio window. If bQuit is TRUE, 
//           PostQuitMessage will be called when the window is closed. 
//           Therefore, the stdio window can be used for the main 
//           application window.
// RETURNS:  Handle to window created. 
//--------------------------------------------------------------------------
HWND wopen( HWND hWndParent, HANDLE hInstance, BOOL bQuit )
{
  HWND hWnd;

  hWnd = CreateWindow( ( LPSTR ) "Stdio",
                       ( LPSTR ) "STDIO",
                       WS_OVERLAPPEDWINDOW | WS_VSCROLL,
                       CW_USEDEFAULT,
                       CW_USEDEFAULT,
                       CW_USEDEFAULT,
                       CW_USEDEFAULT,
                       ( HWND )  hWndParent,
                       ( HMENU ) NULL,       
                       ( HANDLE )hInstance, 
                       MAKEINTRESOURCE( DEFSCRBUF ) );       
  if ( hWnd == NULL )
    return ( FALSE );

  ShowWindow( hWnd, SW_SHOW );
  UpdateWindow( hWnd );

  bStdioQuit = bQuit;

  if ( SetStdioData( hWnd, FALSE ) )
    return ( FALSE );

  return ( hWnd );
}

BOOL FAR PASCAL wputsfocus( LPSTR szStr )
{
  return( wputs( GetFocus( ), szStr ) );
}

//--------------------------------------------------------------------------
// FUNCTION: wputs(LPSTR)
// PURPOSE:  Equivalent to puts() stdio function. Currently, '\n' is
//           not recognized as in '\r\n', as with normal puts(). Must 
//           send '\r\n' explicitly.
// RETURNS:  Status of wopen(), if called, otherwise TRUE.
//--------------------------------------------------------------------------
BOOL FAR PASCAL wputs( HWND hWnd, LPSTR szStr )
{
  HDC hDC;
  int nStrLen = lstrlen( szStr );

  if ( !hWnd )
    return ( FALSE );

  // ------------------------------------------------------------------------
  // The value hWndStdio is initialized in this function.
  // ------------------------------------------------------------------------
  if ( GetStdioData( hWnd ) )
    return ( FALSE );

  if ( !( sScrBuf = ( SCRBUF ) GlobalLock( hScrBuf ) ) )
  {
    WRError( hWndStdio, "wputs", IS_NOLOCK );
    return ( FALSE );
  }

  hDC = GetDC( hWndStdio );
  SetupStdioDC( hDC );
  PutStdio( hDC, nStrLen, ( LPSTR ) szStr );
  ReleaseDC( hWndStdio, hDC );

  SetStdioData( hWndStdio, FALSE );

  GlobalUnlock( hScrBuf );

  return ( TRUE );
} 

//--------------------------------------------------------------------------
// FUNCTION: CreateStdioWindow(LPSTR, DWORD, int, int, int, int, HWND, 
//                             HANDLE, BOOL)
// PURPOSE:  Create an I/O window with definable name, style, size, etc.
// RETURNS:  Handle to window created. 
//--------------------------------------------------------------------------
HWND FAR PASCAL CreateStdioWindow( LPSTR lpszWindowName, DWORD dwStyle,
                       int X, int Y, int nWidth, int nHeight,
                       HWND hWndParent, HANDLE hInstance, BOOL bQuit, WORD wScrBuf )
{
  HWND hWnd;

  hWnd = CreateWindow( ( LPSTR ) "Stdio",
                       ( LPSTR ) lpszWindowName,
                       dwStyle,
                       X,
                       Y,
                       nWidth,
                       nHeight,
                       ( HWND )   hWndParent,
                       ( HMENU )  NULL,       
                       ( HANDLE ) hInstance, 
                       MAKEINTRESOURCE( wScrBuf ) );

  if ( hWnd == NULL )
    return ( FALSE );

  bStdioQuit = bQuit;

  if ( SetStdioData( hWnd, FALSE ) )
    return ( FALSE );

  return ( hWnd );
}

DWORD _fastcall StdioVScroll( WORD wParam, LONG lParam )
{
  short nVScrollInc;
  RECT rcRect;

  GetClientRect( hWndStdio, &rcRect );

  switch ( wParam )
  {
    case SB_LINEDOWN:
      nVScrollInc = 1;
      break;

    case SB_LINEUP:
      nVScrollInc = -1;
      break;

    case SB_PAGEUP:
      nVScrollInc = min( -1, -rcRect.bottom / nStdioCharHeight );
      break;

    case SB_PAGEDOWN:
      nVScrollInc = max( 1, rcRect.bottom / nStdioCharHeight );
      break;

    case SB_CLEAR:
      // Fall through:

    case SB_BOTTOM:
      nVScrollInc = iVScrollMax - iVScrollPos;
      break;

    case SB_TOP:
      nVScrollInc = -iVScrollPos;
      break;

    case SB_THUMBPOSITION:
      nVScrollInc = LOWORD( lParam ) - iVScrollPos;
      break;

    default:
      nVScrollInc = 0;
  }
  //------------------------------------------------------------------------
  // min = Incrementation should never exceed movement from current position
  //       to maximum scrolling position when moving down.  This could only 
  //       occur when the viewing window is resized from small to large 
  //       and the maximum scroll is decreased in the window enlargement 
  //       and the current scrolling position exceeds the new max.
  // max = Incrementation should never me any smaller than movment from
  //       current position to minimum scrolling position when moving up.
  //       This could only occur if window is resized from small to large
  //       and the maximum scroll is decreased when current position is 
  //       around the top of the image.
  //------------------------------------------------------------------------
  if ( nVScrollInc =
       max( -iVScrollPos, min( nVScrollInc, iVScrollMax - iVScrollPos ) ) )
  {
    iVScrollPos += nVScrollInc;
    if ( wParam != SB_CLEAR )
      StdioUpdate( nVScrollInc, FALSE );
  }

  return ( iVScrollPos );
}

int _fastcall StdioKeyBoard( WORD wParam, LONG lParam )
{
  switch ( wParam )
  {
    case VK_HOME:
      SendMessage( hWndStdio, WM_VSCROLL, SB_TOP, 0L );
      break;

    case VK_END:
      SendMessage( hWndStdio, WM_VSCROLL, SB_BOTTOM, 0L );
      break;

    case VK_PRIOR:
      SendMessage( hWndStdio, WM_VSCROLL, SB_PAGEUP, 0L );
      break;

    case VK_NEXT:
      SendMessage( hWndStdio, WM_VSCROLL, SB_PAGEDOWN, 0L );
      break;

    case VK_UP:
      SendMessage( hWndStdio, WM_VSCROLL, SB_LINEUP, 0L );
      break;

    case VK_DOWN:
      SendMessage( hWndStdio, WM_VSCROLL, SB_LINEDOWN, 0L );
      break;

    default:
      return ( 0 );
  }
  return ( 1 );
}

#pragma optimize("egl",off)     // Temporarily deactivates global optimizations.
//--------------------------------------------------------------------------
// FUNCTION: StdioWndProc(HWND, unsigned, WORD, LONG)
// PURPOSE:  Process messages for the I/O window. This function should 
//           be exported in the application's .DEF file. 
//--------------------------------------------------------------------------
LONG FAR PASCAL StdioWndProc( HWND hWnd, WORD wMessage, WORD wParam, LONG lParam )
{
  LPPOINT ptMinMaxInfo;

  switch ( wMessage )
  {
    case WM_CREATE:

      BUFSIZE = LOWORD( ( LONG ) ( ( ( LPCREATESTRUCT ) lParam )->lpCreateParams ) );
      hScrBuf = nCurrPos = 0;
      hWndStdio = hWnd;

      // Initialize stdio variables.
      StdioStart( hWnd );

      if ( SetStdioMem( hWnd, BUFSIZE ) )
        return ( FALSE );

      // Save this instance of those variables.
      SetStdioData( hWnd, TRUE );

      break;

    case WM_SIZE:
    {
      RECT rcNewRect;

      GetStdioData( hWnd );
      GetClientRect( hWnd, &rcNewRect );  // Update client coordinates.
      bMinMax = ( rcNewRect.bottom / nStdioCharHeight >= MAXSCRLINES );

      // Prevent repainting of scroll bar which causes resizing of client area.
      if ( !bScrollPainted )
      {
        if ( bScrollEnabled )
        {
          iVScrollMax = max( 0, ( bMinMax ? iVScrLineCnt + 1 : iVScrLineCnt )
                                     - rcNewRect.bottom / nStdioCharHeight );         
          iVScrollPos = min( iVScrollPos, iVScrollMax );
        }
        StdioUpdate( 0, FALSE ); // generates WM_SIZE.
      }
      SetStdioData( hWnd, FALSE );
    }
    break;

//    case WM_SYSCOLORCHANGE:
//      return ( DefWindowProc( hWnd, wMessage, wParam, lParam ) );
//      break;

    case WM_GETMINMAXINFO:
    {
      WORD wXMax;
      WORD wYMax;

      if ( !nStdioCharWidth )
      {
        HDC  hDC;
        TEXTMETRIC Metrics;

        hDC = CreateIC( "DISPLAY", NULL, NULL, NULL );
        GetTextMetrics( hDC, ( LPTEXTMETRIC ) &Metrics );
        ReleaseDC( hWnd, hDC );

        nStdioCharWidth = Metrics.tmMaxCharWidth;
        nStdioCharHeight = Metrics.tmHeight + Metrics.tmExternalLeading;
      }

      wXMax = nStdioCharWidth * ( MAXLINELEN - 1 ) + 2 *
              GetSystemMetrics( SM_CXFRAME );
      wYMax = nStdioCharHeight * MAXSCRAREA + 2 *
              GetSystemMetrics( SM_CYFRAME ) + GetSystemMetrics( SM_CYMENU ) +
              GetSystemMetrics( SM_CYCAPTION );

      // Constrain the sizing of the window to 80 by 25 characters.
      ptMinMaxInfo = ( LPPOINT ) lParam;
      ptMinMaxInfo[ 1 ].x = wXMax;
      ptMinMaxInfo[ 1 ].y = wYMax;
      ptMinMaxInfo[ 4 ].x = wXMax;
      ptMinMaxInfo[ 4 ].y = wYMax;
                          
      break;
    }

    case WM_PAINT:
      // Repaint the Stdio window
      GetStdioData( hWnd );
      StdioPaint( );
      break;

    case WM_VSCROLL:
      GetStdioData( hWnd );
      StdioVScroll( wParam, lParam );
      SetStdioData( hWnd, FALSE );
      break;

    case WM_KEYDOWN:
      GetStdioData( hWnd );
      if ( !StdioKeyBoard( wParam, lParam ) )
        return ( DefWindowProc( hWnd, wMessage, wParam, lParam ) );
      SetStdioData( hWnd, FALSE );
      break;

    case WM_DESTROY:
      // --------------------------------------------------------------------
      // If specified when created, PostQuitMessage should be called
      // when the window is destroyed.
      // --------------------------------------------------------------------
      GetStdioData( hWnd );
      GlobalFree( hScrBuf );      // Free memory used for display in Stdio.
      GlobalFree( GET_STDIOHND ); // Free memory referenced by session handle.
      SET_STDIOHND( 0 );          // Clear Stdio session handle.

      if( bStdioQuit )
        PostQuitMessage( 0 );
      break;

    case WM_CLOSE:
      // Go ahead and close down
      // -- fall through to default --
    default:
      return ( DefWindowProc( hWnd, wMessage, wParam, lParam ) );
      break;
  }
  return( 0L );
}
#pragma optimize("",on)  // Restores global optimization settings from CL.             


