/*
***************************************************************************
**              ROCKWELL CONFIDENTIAL AND PROPRIETARY
**        This source is the sole property of ROCKWELL Inc.
**    Reproduction or utilization of this source in whole or in 
**  part is forbidden without the written consent of ROCKWELL Inc.
***************************************************************************
**                 (c) Copyright ROCKWELL, INC. 1991
**                       All Rights Reserved
***************************************************************************
*/

/* 
** Last update:   
**	Last PVCS update: $Date:   12 Aug 1992 19:59:34  $ 
*/

/*
**	TITLE:		rawsess.c
**
**	FACILITY:
**   Windows Developer Functions used in the API DLL and defined in
**   the technical manual as Session Functions.
**
**	MODULE DESCRIPTION:
**   The functions is this module are used to open and close an API 
**   session.  Once opened, an API session may not be opened by another
**   application or application instance.  The function RFCloseSess
**   must be called to close the currently open session to allow access
**   by another application.  These functions should NOT require 
**   modification for implementation in an existing Windows application.
**
**   These functions are NOT called by the API and may be given any 
**   desired name.  They are, however, defined in the Technical Manual 
**   with the names used in this source module.
*/

/*
**
**	Revision History:
**
**		$Log:   D:/api_lib/rawsess.c_v  $
 * 
 *    Rev 1.0   12 Aug 1992 19:59:34   Robert Chen
 * Initial revision.
** 
**    Rev 1.2   17 Oct 1991 10:41:22   JIM
** Global update of T.30/T.4 API source for version 1.00.
** 
**    Rev 1.1   12 Sep 1991 11:18:48   JIM
** Global update of T.30/T.4 API source for version 0.22B.
** 
**    Rev 1.0   08 Aug 1991 18:47:58   JIM
** Update to facilitate single session DLL version of API.
** 
*/

#include <windows.h>
#include "rawglobl.h"
#include "rawerr.h"
#include "rawclas1.h"
#include "zapicls1.h"
#include "fapi.h"
#include "zapi.h"

// Variables global to this module:
WORD wSessCnt = 0;  // Session Count used to prevent the opening of 

//---------------------------------------------------------------------------
// Function:  RFOpenSess - Open API session in preparation for a fax
//                         transaction (Developer Function).
// Author:    
//   James Y. Wilson
// Description:     
//   This function initializes an API session by checking the COMx
//   port and calling the RZSetExtData to transfer the contents of the
//   user defined data structure to local memory.  The Session Count
//   variable is incremented to prevent other applications from 
//   opening sessions.
// Args:
//   wComPort - 
//     Indicates the id of the COMx port to be used by the Serial I/O
//     developer functions.  Valid values are 0-3 for COM1 - COM4.         
//   lpExtDataParm -
//     Points to the data to be later type cast for use in the Serial I/O
//     developer functions.
// Return:
//   Returns RF_STATUS_NONE (defined to be 0) for success.  Any other value
//   indicates an error.
//---------------------------------------------------------------------------
int FAR PASCAL RFOpenSess( WORD wComPort, LONG lpExtDataParm )
{
  int iStatus;

  if ( wSessCnt )
  {
    WRError( GetFocus( ), "RFOpenSession: wSessCnt Error", IS_SESSIONINUSE );
    return ( RF_ERR_GEN_SESSINUSE );
  }

  // ------------------------------------------------------------------------
  // Check for empty context area.
  // ------------------------------------------------------------------------
  if ( !RFRetContextPtr( ) )
  {
    WRError( GetFocus( ), "RFOpenSession: RFRetContextPtr Error", 
             IS_SESSIONNOMEM );
	 return ( RF_ERR_GEN_INVCONTXT );
  }

  // ------------------------------------------------------------------------
  // Check for unsupported or unavailable COM port.
  // ------------------------------------------------------------------------
  iStatus = CheckCom( wComPort );   
  if ( iStatus < 0 && iStatus != IE_OPEN )
  {
    WRError( GetFocus( ), "RFOpenSession: CheckCom Error", iStatus );
    return ( RF_ERR_DEV_INV_COM_ID );
  }

  // ------------------------------------------------------------------------
  // Check for valid initialization of the Extended Data area in the developer
  // DLL.
  // ------------------------------------------------------------------------
  if ( RZSetExtData( wComPort, lpExtDataParm ) )
    return ( RF_ERR_GEN_EXTDATA );

  pExtData = (LPEXTDATA) RZGetExtData();

  RFInit( );  
  ++wSessCnt;

  return ( RF_STATUS_NONE );
}

//---------------------------------------------------------------------------
// Function: RFCloseSess - Close an API session previously opened by
//                         RFOpenSess (Developer Function).
// Author:
//   James Y. Wilson
// Description:
//   This function closes a previously opened API session to allow access
//   by other applications or application instances.
// Args:
//   No arguments required.
// Return:
//   This function returns RF_STATUS_NONE (defined to be zero) for success. 
//   The value RF_ERR_GEN_SESSNOUSE is returned if a session has not been
//   previously opened.  The value RF_ERR_GEN_INVCONTXT is returned if  
//   the user defined data structure is not successfully cleared.
//---------------------------------------------------------------------------
int FAR PASCAL RFCloseSess( void )
{
  // ------------------------------------------------------------------------
  // Check that context area IS in use.
  // ------------------------------------------------------------------------
  if ( !wSessCnt )
  {
    WRError( GetFocus( ), "RFCloseSession: RFRetContextPtr Error",
             IS_SESSIONNOUSE );
    return ( RF_ERR_GEN_SESSNOUSE );
  }

  RFClose( );

  // ------------------------------------------------------------------------
  // Check that local ExtData memory is correctly freed.  WRError called
  // at lower level.
  // ------------------------------------------------------------------------
  if ( RZSetExtData( 0, ( LONG ) NULL ) )
    return ( RF_ERR_GEN_INVCONTXT );

  --wSessCnt;

  return ( RF_STATUS_NONE );
}

//---------------------------------------------------------------------------
// Function: RFCheckPollPassword - Verify the password in secured polling.
//
// Author:
//   James Y. Wilson
// Description:
//   This function determines the password received in secured polling and
//   compares it to a character string maintained in the EXTDATA structure.
// Args:
//   No arguments required.
// Return:
//   A return value of 0 indicates a valid password, while a nonzero value
//   indicates an invalid password.
//---------------------------------------------------------------------------  
int FAR PASCAL RFCheckPollPassword( void )
{
  return ( 0 );  // Polling is NOT supported in the current version of the
}                // test harness, WAPIMAIN.EXE


int FAR PASCAL RFGetSessCnt( void )
{
   return wSessCnt;
}
