/*
***************************************************************************
**              ROCKWELL CONFIDENTIAL AND PROPRIETARY
**        This source is the sole property of ROCKWELL Inc.
**    Reproduction or utilization of this source in whole or in 
**  part is forbidden without the written consent of ROCKWELL Inc.
***************************************************************************
**                 (c) Copyright ROCKWELL, INC. 1991
**                       All Rights Reserved
***************************************************************************
*/

/* 
** Last update:   
**	Last PVCS update: $Date:   12 Aug 1992 19:59:44  $ 
*/

/*
**	TITLE:		rawdsply.c
**
**	FACILITY:
**   Developer Functions of the Reporting Class
**
**	MODULE DESCRIPTION:
**   The functions in this module demonstrate the uses of the API Developer
**   Functions defined to allow session progress reports and user feedback.
**   In its current form, this implmentation requires the use of the 
**   generalized output function "wputs" residing in the DLL, RAWCLAS1.DLL.
**   
**   Since these functions are called by the API, they must be minimally 
**   defined as empty code blocks to allow successful linkage of the 
**   APICLAS1.DLL.
*/

/*
**	Revision History:
**
**		$Log:   D:/api_lib/rawvdspl.c_v  $
 * 
 *    Rev 1.0   12 Aug 1992 19:59:44   Robert Chen
 * Initial revision.
** 
**    Rev 1.4   17 Oct 1991 10:40:46   JIM
** Global update of T.30/T.4 API source for version 1.00.
** 
**    Rev 1.3   12 Sep 1991 11:18:06   JIM
** Global update of T.30/T.4 API source for version 0.22B.
** 
**    Rev 1.2   08 Aug 1991 18:47:14   JIM
** Update to facilitate single session DLL version of API.
** 
**    Rev 1.1   20 May 1991 09:14:18   JIM
** 
**    Rev 1.0   13 May 1991 10:02:50   JIM
** Initial Revision
** 
*/
#include <windows.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h> 
#include "rawglobl.h"
#include "rawsdio.h"
#include "zapi.h"
#include "zapicls1.h"
#include "vapi.h"

#define fitoa( a, b, c ) ( ( LPSTR ) itoa( a, b, c ) )

//---------------------------------------------------------------------------
// Function:  RVEndTransReport - Called by the API to report the end of
//                               a fax transaction.
// Author:
//   James Y. Wilson
// Description:
//   This function conforms to the specification presented in the Reporting
//   functions section of the Technical Manual.  A formatted message,
//   containing the values of the function arguments, is displayed using
//   using the utillity function "wputs".  This generalized output function
//   resides in the DLL RAWCLAS1.DLL.
// Args:
//   iStatus -   The code describing a condition of success or failure
//               in the overall transaction processing.
//   iCodeLine - The line of code where the error occurred.
//   szFile -    Points to a character string naming the file where the error
//               occurred.
//   bAbort -    Boolean value where TRUE indicates that the transaction
//               was ended due to a user initiated termination, and FALSE 
//               where the ending was caused by a set of circumstances not 
//               under user control.
// Return:
//   No return value.
//---------------------------------------------------------------------------
VOID FAR PASCAL RVEndTransReport( int iStatus, int iCodeLine, LPUSTR szFile, BOOL bAbort )
{
  HWND hWnd;

  #if ( WIN_TEST )
  FooTimer( ON, "RVEndTransReport" );
  #endif

  // ------------------------------------------------------------------------
  // Check value of pExtData.
  // This will allow reporting of RFOpenSess errors which may have prevented
  // the call to the SetExtData function which performs Extended Data
  // initialization.
  // ------------------------------------------------------------------------
  if ( pExtData )
    hWnd = pExtData->hWndMain;
  else
    hWnd = GetFocus( );

  wsprintf( szTmpStr0, "\n-- RVEndTransReport --\n\
Status Code: %s\n\
Line of Code: %s\n\
Module Name: %s\n\
User Initiated Abort: %s\n\n",
           fitoa( iStatus, szTmpStr1, 10 ), fitoa( iCodeLine, szTmpStr2, 10 ),
           ( LPSTR ) szFile, ( LPSTR ) ( bAbort ? "YES" : "NO" ) );

  #pragma message( "WARNING: 'wputs' used for application output, File: rawdsply.c" )
  wputs( hWnd, szTmpStr0 );

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif
}

//---------------------------------------------------------------------------
// Function:  RVWarnReport - Called by the API to report the warning
//                           described by the function arguments.
// Author:
//   James Y. Wilson
// Description:
//   This function conforms to the specification presented in the Reporting
//   functions section of the Technical Manual.  A formatted message,
//   containing the values of the function arguments, is displayed using
//   using the utillity function "wputs".  This generalized output function
//   resides in the DLL RAWCLAS1.DLL.
// Args:
//   iErrCode -  The error code generated by the API.
//   iCodeLine - The line of code where the error occurred.
//   szFile -    Points to a character string naming the file where the error
//               occurred.
// Return:
//   No return value.
//---------------------------------------------------------------------------
void FAR PASCAL RVWarnReport( int iErrCode, int iCodeLine, LPUSTR szFile )
{
  HWND hWnd;

  #if ( WIN_TEST )
  FooTimer( ON, "RVWarnReport" );
  #endif

  // ------------------------------------------------------------------------
  // Check value of pExtData.
  // This will allow reporting of RFOpenSess errors which may have prevented
  // the call to the SetExtData function which performs Extended Data
  // initialization.
  // ------------------------------------------------------------------------
  if ( pExtData )
    hWnd = pExtData->hWndMain;
  else
    hWnd = GetFocus( );

  // Verify that warning report log output is enabled.
  if ( IsLogStat( WARNLOGOUT ) )
  {
    wsprintf( szTmpStr, "Warning - Code: %s Line: %s Module: %s\n",
              fitoa( iErrCode, szTmpStr1, 10 ), fitoa( iCodeLine, szTmpStr2, 10 ),
              ( LPSTR ) szFile );

    #pragma message( "WARNING: 'wputs' used for application output, File: rawdsply.c" )
    wputs( hWnd, szTmpStr );
  }

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif
}

#if defined ( MSC600 )
  #pragma optimize ( "egl", off )
#endif
#if defined ( MSC600 )
  #pragma optimize ( "", on )
#endif

//--------------------------------------------------------------------------- 
// Function RVShowTxAt - Called by the API to display the DCE echo of 
//                       transmitted AT commands.
// Author:
//   James Y. Wilson
// Description:
//   This function is called by the API to allow the option of displaying
//   those AT commands about to be transmitted to the DCE.  This allows
//   the developer to create a more interactive terminal like display.
// Args:
//   sPtr - Points to the character string containing the AT command to
//          be formatted and printed.
// Return: 
//   No return value.
//--------------------------------------------------------------------------- 
void FAR PASCAL RVShowTxAt( LPUSTR sPtr )
{                     
  // Verify that AT command log output is enabled.
  if ( !( IsLogStat( ATCMLOGOUT ) ) )
    return;

  wsprintf( szTmpStr, "Tx: %s\n", ( LPSTR ) sPtr );

  #pragma message( "WARNING: 'wputs' used for application output, File: rawdsply.c" )
  wputs( pExtData->hWndMain, szTmpStr );          // Direct output to current MS Window.
}

//---------------------------------------------------------------------------
// Function:  RVShowRxAt - Called by the API to display the DCE response to
//                         transmitted AT commands.
// Author:
//   James Y. Wilson
// Description:
//   This function is called by the API to display the character string 
//   transmitted by the DCE to the DTE in command mode.
// Args:
//   sPtr - points to the DCE originated character string to be printed
//          and formatted.
// Return:
//   No return value.
//---------------------------------------------------------------------------
void FAR PASCAL RVShowRxAt( LPUSTR sPtr )
{
  // Verify that AT command log output is enabled.
  if ( !( IsLogStat( ATCMLOGOUT ) ) )
    return;

  wsprintf( szTmpStr, "Rx: %s\n", ( LPSTR ) sPtr );

  #pragma message( "WARNING: 'wputs' used for application output, File: rawdsply.c" )
  wputs( pExtData->hWndMain, szTmpStr );          // Direct output to current MS Window.
}


