/*
***************************************************************************
**              ROCKWELL CONFIDENTIAL AND PROPRIETARY
**        This source is the sole property of ROCKWELL Inc.
**    Reproduction or utilization of this source in whole or in 
**  part is forbidden without the written consent of ROCKWELL Inc.
***************************************************************************
**                 (c) Copyright ROCKWELL, INC. 1991
**                       All Rights Reserved
***************************************************************************
*/

/* 
** Last update:   
**	Last PVCS update: $Date:   12 Aug 1992 19:59:58  $ 
*/

/*
**	TITLE:		rawautl1.c
**
**	FACILITY:
**   Developer Functions
**
**	MODULE DESCRIPTION:
**   The functions in this module control the incrementation of the
**   API timers, allocate global memory to save the contents of
**   EXTDATA structure, and lock/unlock the memory allocated for the
**   ASCII files transmitted from memory.
**
*/

/*
**
**	Revision History:
**
**		$Log:   D:/api_lib/rawvutl1.c_v  $
 * 
 *    Rev 1.0   12 Aug 1992 19:59:58   Robert Chen
 * Initial revision.
** 
**    Rev 1.4   17 Oct 1991 10:40:12   JIM
** Global update of T.30/T.4 API source for version 1.00.
** 
**    Rev 1.3   12 Sep 1991 11:17:30   JIM
** Global update of T.30/T.4 API source for version 0.22B.
** 
**    Rev 1.2   08 Aug 1991 18:46:28   JIM
** Update to facilitate single session DLL version of API.
** 
**    Rev 1.1   20 May 1991 09:14:36   JIM
** 
**    Rev 1.0   13 May 1991 09:46:56   JIM
** Initial Revision
** 
** 
*/

#include <windows.h>
#include <stdlib.h>  
#include <bios.h>
#include <sys\timeb.h>
#include "rawglobl.h"
#include "rawsdio.h"
#include "rawerr.h"
#include "zapi.h"
#include "vapi.h"
#include "zapidos.h"
#include "zapicls1.h"

struct timeb tbVTimeStart, tbVTimeEnd;

//---------------------------------------------------------------------------
// Function:  RVPollSysTimerInit - Initializes the static memory containing
//                                 the value of the last timer interval.
// Author:
//   James Y. Wilson
// Description:
//   This function initializes the EXTDATA field containing the value
//   of the last timer interval.  Other members of the structure EXTDATA
//   are initialized using the RVSetExtData function, called from the
//   Developer Function, RVOpenSess.
// Args:
//   No arguments necessary.
// Return:
//   No return value.
//---------------------------------------------------------------------------
VOID FAR PASCAL RVPollSysTimerInit( void )
{
  ftime( &tbVTimeStart );
  pExtData->tbTimeStart = tbVTimeStart;
}

//---------------------------------------------------------------------------
// Function:  RVPollSysTimer - Calculates the amount of time passed and
//                             calls the API function which updates its
//                             internal timers.
// Author:
//   James Y. Wilson
// Description:
//   This function calculates the amount of time passed since the last call
//   and saves the value of the current time in an EXTDATA field.  The
//   result of this calculation is passed to the API function which 
//   increments internal API system timers.
// Args:
//   No arguments necessary.
// Return:
//   No return value.
//---------------------------------------------------------------------------
VOID FAR PASCAL RVPollSysTimer( void )
{
  LONG lDuration;
  
  tbVTimeStart = pExtData->tbTimeStart;
  ftime( &tbVTimeEnd );
  lDuration = ( ( 1000 * ( tbVTimeEnd.time - tbVTimeStart.time ) ) +
              tbVTimeEnd.millitm - tbVTimeStart.millitm ); 
  RVUpdateTimers( ( unsigned int ) lDuration );
  pExtData->tbTimeStart = tbVTimeEnd;
}

//---------------------------------------------------------------------------
// Function:  RVGetExtData - Retrieves the pointer to the EXTDATA structure
//                           previously initialized.
// Author:
//   James Y. Wilson
// Description:
//   This function allows access to the data within the EXTDATA structure
//   from within outside code segments.  It is recommended, however, that
//   the data of this user defined structure be updated using the
//   RVSetExtData structure and that use of this function be limited
//   to retrieving the required values.
// Args:
//   No arguments required.
// Return:
//   No return value.
//---------------------------------------------------------------------------
// LONG FAR PASCAL RVGetExtData( void )
// {
//   //RC_MOD
//   return ( ( LONG ) ( ( LPEXTDATA ) pExtData ) );
// }

//---------------------------------------------------------------------------
// Function:  RVSetExtData - Set or updates the values of a user defined
//                           data structure.
// Author:
//   James Y. Wilson
// Description:
//   This function initializes the values of the EXTDATA structure.  It may
//   also be used to update the existing values due to a change by the
//   end user.  The EXTDATA structure, as explained in the Technical Manual,
//   was defined to contain COMx port configuration parameters.  The
//   developer may which to adopt a similar scheme using a different
//   structure, defined in a header file for this module and the application
//   module.
// Args:
//   wComPort - The COMx port, beginning with 0, to use in future calls to the Serial
//     I/O Developer Functions. 
//   lpExtData - Long pointer to the structure which will later be type cast
//     and retained in locally allocated memory.
// Return:
//   A return value of zero indicates success.  Any other values indicate
//   an unspecified failure.
//---------------------------------------------------------------------------
// int FAR PASCAL RVSetExtData( WORD wComPort, LONG lpExtData )
// {
//   // ------------------------------------------------------------------------
//   // Free locally allocated EXTDATA structure.
//   // ------------------------------------------------------------------------
//   if ( !lpExtData && pExtData )
//   {
//     if ( LocalFree( ( LOCALHANDLE ) pExtData ) )
//     {
//       WRError( GetFocus( ), "SetExtData: LocalFree Error", IS_NOFREE );
//       return ( -1 );
//     }
//     pExtData = NULL;
//     pCommDCB = NULL;
//     return ( 0 );    // Local memory of EXTDATA successfully freed.
//   }
// 
//   if ( !pExtData )
//   {
//     if ( !( pExtData = ( NPEXTDATA ) LocalAlloc( LPTR, sizeof( EXTDATA ) ) ) )
//     {
//       WRError( GetFocus(), "SetExtData: LocalAlloc Error", IS_OUTOFLOCALMEM );
//       return ( -1 );
//     }
// 
//   // Testing:
//     *pExtData = *( ( LPEXTDATA ) lpExtData );
//   }
// 
//   // -----------------------------------------------------------------------
//   // Copy data pointed to by lpExtData to allocated pExtData.
//   // -----------------------------------------------------------------------
//   // Testing:
// //  *pExtData = *( ( LPEXTDATA ) lpExtData );
//   pExtData->wComPort = wComPort;
// 
//   // -----------------------------------------------------------------------
//   // Do not update pCommDCB unless it is being initialized for the first 
//   // time.
//   // -----------------------------------------------------------------------
//   if ( !pCommDCB )
//     pCommDCB = &( pExtData->CommDCB );
// 
//   return ( 0 );      // Local memory of EXTDATA successfully allocated.
// } 
// 
// 
