/*
***************************************************************************
**              ROCKWELL CONFIDENTIAL AND PROPRIETARY
**        This source is the sole property of ROCKWELL Inc.
**    Reproduction or utilization of this source in whole or in 
**  part is forbidden without the written consent of ROCKWELL Inc.
***************************************************************************
**                 (c) Copyright ROCKWELL, INC. 1991
**                       All Rights Reserved
***************************************************************************
*/

/* 
** Last update:   
**	Last PVCS update: $Date:   12 Aug 1992 20:00:02  $ 
*/

/*
**	TITLE:		rawwsio.c
**
**	FACILITY:
**   Windows Developer Functions used in the API DLL and defined in
**   the technical manual as Serial I/O Developer Functions.
**
**	MODULE DESCRIPTION:
**   The functions in this module call the Microsoft Windows SDK, System
**   Service Communication Routines to perform basic serial I/O.  If
**   a custom communications driver is used these functions should be
**   modified to conform to the existing application standard.  Otherwise,
**   the functions in this module should NOT require modification.
**
**   These functions ARE called by the API and must be named as specified in
**   the following source.  The return values of each function as defined in 
**   the current source must be preserved.
*/

/*
**	Revision History:
**
**		$Log:   D:/api_lib/rawwsio.c_v  $
 * 
 *    Rev 1.0   12 Aug 1992 20:00:02   Robert Chen
 * Initial revision.
** 
**    Rev 1.4   17 Oct 1991 10:41:30   JIM
** Global update of T.30/T.4 API source for version 1.00.
** 
**    Rev 1.3   12 Sep 1991 11:18:54   JIM
** Global update of T.30/T.4 API source for version 0.22B.
** 
**    Rev 1.2   08 Aug 1991 18:48:04   JIM
** Update to facilitate single session DLL version of API.
** 
**    Rev 1.0   13 May 1991 09:42:46   JIM
** Initial Revision
*/

/*--------------------------------------------------------------------------
*                       Table of SDK Function Usage
*
* Developer Function                    MS Windows Function
* 
* RZOpenCom                             BuildCommDCB
*                                        OpenComm
* 
* RZCloseCom                            CloseComm
* 
* RZWriteCom                            WriteComm
*                                        GetCommError
* 
* RZReadCom                             ReadComm
*                                        GetCommError
* 
* RZComTxBufRoom                        GetCommError
* RZComTxBufEmpty                         
* RZComRxDataAvail
* 
* RZEnaXonXoff                          SetCommState
* RZDisXonXoff                     
* RZBaud38400
* RZBaud19200 
* RZBaud2400
* 
* RZFlushTxComm                         FlushComm
* RZFlushRxComm
* RZStartTxHighSpeed 
* RZStartRxHighSpeed
* RZEndHighSpeed
*
\*----------------------------------------------------------------------------*/
#include <windows.h>
#include <stdlib.h>  
#include <stdio.h>   
#include <string.h>  
#include <ctype.h>   
#include "rawglobl.h"
#include "rawsdio.h"
#include "rawerr.h"
#include "zapi.h"
#include "zapicls1.h"

#define ON            TRUE
#define OFF           FALSE

WORD wCBR2400; 
WORD wCBR19200;
WORD wCBR38400;

// Variables global to this module:
COMSTAT CommStat;
char    szComStr[][ 5 ] = { COM1, COM2, COM3, COM4 };

//---------------------------------------------------------------------------
// Function:  RZOpenCom - Opens the COMx port (Developer Function).
// Author:
//   James Y. Wilson
// Description:
//   This function opens the COMx port specified in the user defined data
//   structure, initialized in the RFOpenSess function.  
// Args:
//   No arguments required.
// Return:
//   Returns a zero if the COMx port was successfully opened.  Any other
//   value indicates success.
//---------------------------------------------------------------------------
int FAR PASCAL RZOpenCom( void ) 
{
  static char szCommDriver[ PROFILESTRMAX ];
  static WORD wWinVersion;

  #if ( WIN_TEST )
  FooTimer( ON, "RZOpenCom" );
  #endif

  // RC_MOD
  pExtData = (LPEXTDATA) RZGetExtData();

  if ( InitDCB( pCommDCB, pExtData->wRecvBuf ) )
  {
    WRError( pExtData->hWndMain, "RZOpenCom: InitDCB Error", IS_DCBINITERROR );
    return ( 1 );
  }

  wWinVersion = LOWORD( GetVersion( ) );

  // ------------------------------------------------------------------------
  // Define different baud rate constants for Windows version 3.1 
  // ------------------------------------------------------------------------
  GetPrivateProfileString( "boot", "comm.drv", "", szCommDriver, PROFILESTRMAX, 
                           SYSTEMINI );

  if ( !( _fstrcmp( _fstrupr( szCommDriver ), COMDRIVER ) ) &&
        ( HIBYTE( wWinVersion ) == 10 && LOBYTE( wWinVersion ) == 03 ) )
  {
//    wCBR2400  = 2400;
//    wCBR19200 = 19200;
    wCBR2400  = 0xff14;
    wCBR19200 = 0xff18;
    wCBR38400 = 0xff1b;
  }
  else  
  {
    wCBR2400  = 2400;
    wCBR19200 = 19200;
    wCBR38400 = 38400;
  }

  //------------------------------------------------------------------------
  // Keep trying to Open COM port as long as error is IE_OPEN and user responds
  // with YES.
  //------------------------------------------------------------------------
  do
  {
    if ( ( pExtData->iPortId = OpenComm( szComStr[ pExtData->wComPort ], 
                                         pExtData->wRecvBuf, 
                                         pExtData->wXmitBuf ) ) < 0 )
    {
      LoadString( GetWindowWord( pExtData->hWndMain, GWW_HINSTANCE ), 
                  pExtData->iPortId, szErrStr, ERRSTRMAX );

      if ( pExtData->iPortId == IE_OPEN )
      {
        if ( MessageBox( pExtData->hWndMain, szErrStr, "RZOpenCom: OpenComm Error",
             MB_YESNO | MB_ICONQUESTION ) == IDYES )
          CloseComm( pExtData->wComPort );
        else
          return ( 1 );
      }
      else
      {
        MessageBox( pExtData->hWndMain, szErrStr, "RZOpenCom: OpenComm Error", MB_OK );
        return ( 1 );                // Failed COMx open.
      }
    }
  }
  while ( pExtData->iPortId == IE_OPEN );

  pCommDCB->Id = ( BYTE ) pExtData->iPortId;
  pCommDCB->BaudRate = wCBR2400;
  pCommDCB->fOutxCtsFlow = pCommDCB->fRtsflow = pExtData->bfRtsCts;
  pCommDCB->fOutX = pExtData->bfXonXoff;

  FlushComm( pExtData->iPortId, 0 );         // Flush transmit queue.
  FlushComm( pExtData->iPortId, 1 );         // Flush receive queue.

  if ( UpdateCommState( pCommDCB ) )  // Update COMx device with changes.
  {
    WRError( pExtData->hWndMain, "RZOpenCom: UpdateCommState Error", IS_UPCSERROR );
    return ( 1 );               // Failed COMx open.
  }

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif

  return( 0 );                  // Successful COMx open.
}

//---------------------------------------------------------------------------
// Function:  RZCloseCom - Closes the COMx port (Developer Function).
//
// Author:
//   James Y. Wilson
// Description:
//   This function closes the COMx port opened using the function RZOpenCom.
//   The COMx port Windows id is contained in the user defined data structure
//   passed to the RFOpenSess function.
// Args:
//   No arguments required.
// Return:
//   Returns 0 for success, any other value, such as that returned from the
//   call to the Windows function CloseComm, indicates an error.  Please note
//   that the structure field pExtData->iPortId may contain the error code
//   received when attempting to open the COMx port using the RZOpenCom 
//   function.
//---------------------------------------------------------------------------
int FAR PASCAL RZCloseCom( void )
{
  int iReturn;

  #if ( WIN_TEST )
  FooTimer( ON, "RZCloseCom" );
  #endif

  // ------------------------------------------------------------------------
  // "iPortId" indicates error in RZOpenCom if neg.
  // ------------------------------------------------------------------------
  if ( pExtData->iPortId >= 0 )
  {
    FlushComm( pCommDCB->Id, 0 );
    FlushComm( pCommDCB->Id, 1 );
    iReturn = CloseComm( pCommDCB->Id );
  }
  else
    iReturn = -1;

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif

  return( iReturn );
}    

//---------------------------------------------------------------------------
// Function:  RZWriteCom - Writes a buffer to the serial port specified in
//                         the structure EXTDATA.
// Author:
//   James Y. Wilson
// Description:
//   This function calls the Windows Communication Systerm Interface function
//   to transfer the buffer to the COMx port.  The COMx port id for writing
//   is contained in the EXTDATA structure.  This structure is maintained
//   by the end user application using the RFSetExtData function.  It is
//   is initialized in the RFOpenSess function.
// Args:
//   sBuff - Points to the character string to be written to the COMx port.
//   cbSize - Specifies the size of the buffer in sBuff.
// Return:
//   A zero value indicates that no data was transmitted.  A positive value
//   indicates the number of bytes successfully transmitted.
//---------------------------------------------------------------------------
int FAR PASCAL RZWriteCom( unsigned char far *sBuff, int cbSize )
{
  int iReturn;

  #if ( WIN_TEST )
  FooTimer( ON, "RZWriteCom" );
  #endif

  if ( CheckCommError( &CommStat, __LINE__ ) )  
    return ( 0 );                               // Fill CommStat buffer.
                                                // Return error condition.
                                                // Check for room in tx queue.
  if ( ( pExtData->wXmitBuf - CommStat.cbOutQue ) < ( WORD ) cbSize )
  {
    WRError( pExtData->hWndMain, "RZWriteCom: WriteComm Error", IS_XMITQUEFULL );
    return ( 0 );                               // Return error condition.
  }

  if ( ( iReturn = WriteComm( pCommDCB->Id, sBuff, cbSize ) ) < 0 ) 
  {
    CheckCommError( &CommStat, __LINE__ ); // Display error.
    return ( 0 );                               // Return error condition.
  }

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif

  return ( iReturn );                           // Return # bytes successfully
}                                               // written.

//---------------------------------------------------------------------------
// Function:  RZReadCom - Read the serial port specified in EXTDATA.
//
// Author:
//   James Y. Wilson
// Description:
//   This function transfers data from the ISR COMx buffer maintained by
//   Windows to the memory pointed to in the parameter sBuff.  The COMx
//   port read is contain in the EXTDATA structure maintained using
//   RFSetExtData.
// Args:
//   sBuff - Points to the memory to be used when obtaining the COMx port
//           data received.
//   cbSize - The size of the buffer pointed to in sBuff.
// Return:
//   A zero value indicates that not data was read.  A positive value
//   inidicates the number of bytes successfully read.
//---------------------------------------------------------------------------
int FAR PASCAL RZReadCom( LPUSTR sBuff, int cbSize )
{
  int iReturn;

  #if ( WIN_TEST )
  FooTimer( ON, "RZReadCom" );
  #endif

  if ( ( iReturn = ReadComm( pCommDCB->Id, sBuff, cbSize ) ) < 0 )
  {
    CheckCommError( &CommStat, __LINE__ );      // Display error.
    return ( 0 );                               // Return error condition.
  }

  #if ( WIN_TEST )
  if ( sBuff[ 0 ] > 128 )
  {
    strcpy( szTmpStr, "Received Data: Not Displayable" );
  }
  else
  {
    wsprintf( szTmpStr, "Received Data: %s\n", ( LPSTR ) sBuff );
  }
  wputs( szTmpStr );
  #endif 

  #if ( WIN_TEST )
  if ( CommStat.cbInQue )
  {
    wsprintf( szTmpStr, "Data Remaining in Read: %i\n", CommStat.cbInQue );
    wputs( szTmpStr );
  }
  #endif

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif

  return ( iReturn );   // This value may be zero if no data to read.
}                       

//---------------------------------------------------------------------------
// Function:  RZComTxBufRoom - Allows the determination of the space
//                             available in the transmit buffer.
// Author:
//   James Y. Wilson
// Description:
//   This function determines the amount of unused space in the Windows 
//   transmit buffer.  It should be used to determine if the block of
//   memory to be written to the COMx port is too large for the unused
//   space available in the transmit buffer.
// Args:
//   No argument required.
// Return:
//   A value of zero indicates that there is no room available in the transmit
//   buffer.  A positive value indicates the number of unused bytes
//   in the transmit buffer.
//---------------------------------------------------------------------------
int FAR PASCAL RZComTxBufRoom( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZComTxBufRoom" );
  #endif

  if ( CheckCommError( &CommStat, __LINE__ ) )  // Fill CommStat buffer.
    return ( 0 );

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif

  return ( pExtData->wXmitBuf - CommStat.cbOutQue );
}

//---------------------------------------------------------------------------
// Function:  RZComTxBufEmpty - Determines the status of the Windows transmit
//                              buffer as empty or not.
// Author:
//   James Y. Wilson
// Description:
//   This function determines whether or not the transmit buffer maintained
//   by Windows is empty.
// Args:
//   No argument is necessary.
// Return:
//   A zero value indicates that the transmit buffer is NOT empty.  A nonzero
//   value indicates that it is.
//---------------------------------------------------------------------------
int FAR PASCAL RZComTxBufEmpty( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZComTxBufEmpty" );
  #endif

  if ( CheckCommError( &CommStat, __LINE__ ) )  // Fill CommStat buffer. 
    return ( 1 );

  #if ( WIN_TEST )
  wsprintf( szTmpStr, "Transmitted Data Characters: %i\n", CommStat.cbOutQue );
  wputs( szTmpStr );
  #endif

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif

  return ( CommStat.cbOutQue == 0 );
}

//---------------------------------------------------------------------------
// Function:  RZComRxDataAvail - Determines if unread data is present in the
//                               receive buffer.
// Author:
//   James Y. Wilson
// Description:
//   This function calls the Windows function which fills the COMMSTAT
//   structure.  The appropriate structure field is used to determine
//   the amount of unread data.
// Args:
//   No argument necessary.
// Return:
//   A zero value indicates that no data has been received.  A positive value
//   indicates the number of bytes received and available for processing.
//---------------------------------------------------------------------------
int FAR PASCAL RZComRxDataAvail( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZComRxDataAvail" );
  #endif

  if ( CheckCommError( &CommStat, __LINE__ ) )  // Fill CommStat buffer.
    return ( 0 );

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif

  return ( CommStat.cbInQue );
}

//---------------------------------------------------------------------------
// Function:  RZEnaXonXoff - Enables XONXOFF flow control if it is not
//                           already enabled.
// Author:
//   James Y. Wilson
// Description:
//   This function enables XONXOFF flow control if requested by a positive
//   iDataMode parameter and XONXOFF and CTS flow control are currently 
//   disabled.  If either XONXOFF or CTS flow control is already enabled the
//   Windows function will NOT be called to avoid the associated overhead.
// Args:
//   iDataMode - a boolean value where TRUE indicates that data mode is 
//               active.
// Return:
//   No return value required.
//---------------------------------------------------------------------------
void FAR PASCAL RZEnaXonXoff( int iDataMode )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZEnaXonXoff" );
  #endif

  if ( iDataMode && !pCommDCB->fOutX && !pCommDCB->fOutxCtsFlow )
  {
    pCommDCB->fOutX = 1; // Enable XON/XOFF flow control used during transmission.
    if ( UpdateCommState( pCommDCB ) )  // Update COMx device with changes.
      WRError( pExtData->hWndMain, "RZEnaXonXoff: UpdateCommState Error", 
               IS_UPCSERROR );
  }
  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif
}

//---------------------------------------------------------------------------
// Function: RZEnaCTS - Enables CTS hardware based flow control if previously
//                      disabled.
// Author:
//   James Y. Wilson
// Description:
//   If CTS and XONXOFF flow control are currently disabled this function 
//   will call the Windows function which enables CTS hardware flow control.
//   If either CTS or XONXOFF flow control is already enabled the
//   Windows function will NOT be called to avoid the associated overhead.
// Args:
//   No argument necessary.
// Return:
//   No return value.
//---------------------------------------------------------------------------
void FAR PASCAL RZEnaCTS( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZEnaCTS" );
  #endif

  if ( !pCommDCB->fOutxCtsFlow && !pCommDCB->fOutX )
  {
    pCommDCB->fOutxCtsFlow = 1;
    pCommDCB->fRtsflow = 1;

    if ( UpdateCommState( pCommDCB ) )  // Update COMx device with changes.
      WRError( pExtData->hWndMain, "RZEnaCTS: UpdateCommState Error", 
               IS_UPCSERROR );
  }
    
  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif
}

//---------------------------------------------------------------------------
// Function: RZDisXonXoff - Disables XONXOFF flow control if previously
//                          enabled.
// Author:
//   James Y. Wilson
// Description:
//   If XONXOFF flow control is currently enabled this function will call the
//   Windows function which disables it.
// Args:
//   No argument necessary.
// Return:
//   No return value.
//---------------------------------------------------------------------------
void FAR PASCAL RZDisXonXoff( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZDisXonXoff" );
  #endif

  if ( pCommDCB->fOutX )
  {
    pCommDCB->fOutX = 0; // Disable XON/XOFF flow control used during transmission.
    pCommDCB->fInX  = 0; // Disable XON/XOFF flow control used during receive.
    if ( UpdateCommState( pCommDCB ) )  // Update COMx device with changes.
      WRError( pExtData->hWndMain, "RZDisXonXoff: UpdateCommState Error", 
               IS_UPCSERROR );
  }

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif
}

//---------------------------------------------------------------------------
// Function: RZDisCTS - Disables CTS hardware based flow control if
//                      previously enabled.
// Author:
//   James Y. Wilson
// Description:
//   If CTS flow control is currently enabled this function will call the
//   Windows function which disables it. 
// Args:
//   No argrment necessary.
// Return:
//   No return value.
//---------------------------------------------------------------------------
void FAR PASCAL RZDisCTS( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZDisCTS" );
  #endif

  if ( pCommDCB->fOutxCtsFlow )
  {
    pCommDCB->fOutxCtsFlow = 0;
    pCommDCB->fRtsflow = 0;
    if ( UpdateCommState( pCommDCB ) )
      WRError( pExtData->hWndMain, "RZDisCTS: UpdateCommState Error",
               IS_UPCSERROR );
  }

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif
}

//---------------------------------------------------------------------------
// Function:  RZBaud38400 - Sets the DTE for high speed communication with the
//                          DCE
// Author:
//   James Y. Wilson
// Description:
//   This function sets the bit rate at 38.4 kbps for use in performing
//   high speed serial I/O with the DCE.
// Args:
//   No arguments required.
// Return:
//   A zero value indicates success in changing the bit rate.  A positive
//   value indicates failure.
//---------------------------------------------------------------------------
int FAR PASCAL RZBaud38400( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZBaud38400" );
  #endif

  pCommDCB->BaudRate = wCBR38400;

  if ( UpdateCommState( pCommDCB ) )  // Update COMx device with changes.
  {
    WRError( pExtData->hWndMain, "RZBaud38400: UpdateCommState Error", 
             IS_UPCSERROR );
    return( 1 );
  }

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif

  return ( 0 );
}  

//---------------------------------------------------------------------------
// Function:  RZBaud19200 - Sets the DTE for high speed communication with the
//                          DCE
// Author:
//   James Y. Wilson
// Description:
//   This function sets the bit rate at 19.2K bps for use in performing
//   high speed serial I/O with the DCE.
// Args:
//   No arguments required.
// Return:
//   A zero value indicates success in changing the bit rate.  A positive
//   value indicates failure.
//---------------------------------------------------------------------------
int FAR PASCAL RZBaud19200( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZBaud19200" );
  #endif

  pCommDCB->BaudRate = wCBR19200;

  if ( UpdateCommState( pCommDCB ) )  // Update COMx device with changes.
  {
    WRError( pExtData->hWndMain, "RZBaud19200: UpdateCommState Error", 
             IS_UPCSERROR );
    return( 1 );
  }

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif

  return ( 0 );
}

//---------------------------------------------------------------------------
// Function: RZBaud2400 - Sets the DTE for low speed communications with the
//                        DCE.
// Author:
//   James Y. Wilson
// Description:
//   This function sets the bit rate to 2.4K bps by calling the appropriate
//   Windows function.
// Args:
//   No argument necessary.
// Return:
//   A zero value indicates success in changing the bit rate.  A positive
//   value indicates failure.
//---------------------------------------------------------------------------
int FAR PASCAL RZBaud2400( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZBaud2400" );
  #endif

  pCommDCB->BaudRate = wCBR2400;

  if ( UpdateCommState( pCommDCB ) )  // Update COMx device with changes.
  {
    WRError( pExtData->hWndMain, "RZBaud2400: UpdateCommState Error", 
             IS_UPCSERROR );
    return ( 1 );
  }

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif

  return ( 0 );
}

//---------------------------------------------------------------------------
// Function:  RZStartRxHighSpeed - Calls the functions required to
//                                 facilitate high speed receive from the
//                                 between the DCE to the DTE.
// Description:
//   As specified in the Technical Manual this function should reapportion
//   the memory designated as a transmit and receive buffer to favor the
//   the receive buffer with more memory.  Since Windows defines the size
//   of the buffers in the call to OpenComm, the sizes may only be changed 
//   by opening and closing the COMx port.  Consequently, the Windows
//   version of this function does not support transmit/receive buffer
//   reapportionment.
// Args:
//   No parameter necessary.
// Return:
//   No return value.
//---------------------------------------------------------------------------
void FAR PASCAL RZStartRxHighSpeed( void ) 
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZStartRxHighSpeed" );
  #endif

  RZFlushRxComm( );
  RZFlushTxComm( );

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif
}

//---------------------------------------------------------------------------
// Function:  RZStartTxHighSpeed - Call the functions required to facilitate
//                                 high speed transmission from the DTE to
//                                 the DCE.
// Author:
//   James Y. Wilson
// Description:
//   This function does not support transmit/receive buffer reapportionment
//   as outlined in the description of the function RZStartRxHighSpeed.  The
//   functions which flush the receive and transmit buffers are called.
// Args:
//   No arguments necessary.
// Return:
//   No return value.
//---------------------------------------------------------------------------
void FAR PASCAL RZStartTxHighSpeed( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZStartTxHighSpeed" );
  #endif

  RZFlushRxComm( );
  RZFlushTxComm( );

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif
}

//---------------------------------------------------------------------------
// Function:  RZEndHighSpeed - Calls the functions required to end
//                             high speed communications between the DCE and
//                             the DTE.
// Author:
//   James Y. Wilson
// Description:
//   This function does not support transmit/receive buffer reapportionment
//   as outlined in the description of the function RZStartRxHighSpeed.  The
//   functions which flush the receive and transmit buffers are called.
// Args:
//   No argument necessary.
// Return:
//   No return value.
// --------------------------------------------------------------------------  
void FAR PASCAL RZEndHighSpeed( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZEndHighSpeed" );
  #endif

  RZFlushRxComm( );
  RZFlushTxComm( );

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif
}

//---------------------------------------------------------------------------
// Function:  RZFlushRxComm - Calls the Windows function which flushes the
//                            receive buffer.
// Author:
//   James Y. Wilson
// Description:
//   This function calls the Windows function which removes the received
//   data from the receive buffer.
// Args:
//   No argument necessary
// Return:
//   No return value.
//---------------------------------------------------------------------------
void FAR PASCAL RZFlushRxComm( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZFlushRxComm" );
  #endif

  if ( FlushComm( pCommDCB->Id, 1 ) < 0 )
    WRError( pExtData->hWndMain, "RZFlushRxComm: FlushComm Error", 
             IS_RXFLUSHERROR );

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif
}

//---------------------------------------------------------------------------
// Function:  RZFlushTxComm - Calls the Windows function which flushes the
//                            transmit buffer.
// Author:
//   James Y. Wilson
// Description:
//   This function calls the Windows function which removes the
//   untransmitted data from the transmit buffer.
// Args:
//   No argument necessary
// Return:
//   No return value.
//---------------------------------------------------------------------------
void FAR PASCAL RZFlushTxComm( void )
{
  #if ( WIN_TEST )
  FooTimer( ON, "RZFlushTxComm" );
  #endif

  if ( FlushComm( pCommDCB->Id, 0 ) < 0 )
    WRError( pExtData->hWndMain, "RZFlushTxComm: FlushComm Error",
             IS_TXFLUSHERROR );

  #if ( WIN_TEST )
  FooTimer( OFF, NULL );
  #endif
}

//---------------------------------------------------------------------------
// Function:  RZGetExtData - Retrieves the pointer to the EXTDATA structure
//                           previously initialized.
// Author:
//   James Y. Wilson
// Description:
//   This function allows access to the data within the EXTDATA structure
//   from within outside code segments.  It is recommended, however, that
//   the data of this user defined structure be updated using the
//   RZSetExtData structure and that use of this function be limited
//   to retrieving the required values.
// Args:
//   No arguments required.
// Return:
//   No return value.
//---------------------------------------------------------------------------
LONG FAR PASCAL RZGetExtData( void )
{
  //RC_MOD
  return ( ( LONG ) ( ( LPEXTDATA ) pExtData ) );
}

//---------------------------------------------------------------------------
// Function:  RZSetExtData - Set or updates the values of a user defined
//                           data structure.
// Author:
//   James Y. Wilson
// Description:
//   This function initializes the values of the EXTDATA structure.  It may
//   also be used to update the existing values due to a change by the
//   end user.  The EXTDATA structure, as explained in the Technical Manual,
//   was defined to contain COMx port configuration parameters.  The
//   developer may which to adopt a similar scheme using a different
//   structure, defined in a header file for this module and the application
//   module.
// Args:
//   wComPort - The COMx port, beginning with 0, to use in future calls to the Serial
//     I/O Developer Functions. 
//   lpExtData - Long pointer to the structure which will later be type cast
//     and retained in locally allocated memory.
// Return:
//   A return value of zero indicates success.  Any other values indicate
//   an unspecified failure.
//---------------------------------------------------------------------------
int FAR PASCAL RZSetExtData( WORD wComPort, LONG lpExtData )
{
  // ------------------------------------------------------------------------
  // Free locally allocated EXTDATA structure.
  // ------------------------------------------------------------------------
  if ( !lpExtData && pExtData )
  {
    if ( LocalFree( ( LOCALHANDLE ) pExtData ) )
    {
      WRError( GetFocus( ), "SetExtData: LocalFree Error", IS_NOFREE );
      return ( -1 );
    }
    pExtData = NULL;
    pCommDCB = NULL;
    return ( 0 );    // Local memory of EXTDATA successfully freed.
  }

  if ( !pExtData )
  {
    if ( !( pExtData = ( NPEXTDATA ) LocalAlloc( LPTR, sizeof( EXTDATA ) ) ) )
    {
      WRError( GetFocus(), "SetExtData: LocalAlloc Error", IS_OUTOFLOCALMEM );
      return ( -1 );
    }

  // Testing:
    *pExtData = *( ( LPEXTDATA ) lpExtData );
  }

  // -----------------------------------------------------------------------
  // Copy data pointed to by lpExtData to allocated pExtData.
  // -----------------------------------------------------------------------
  // Testing:
//  *pExtData = *( ( LPEXTDATA ) lpExtData );
  pExtData->wComPort = wComPort;

  // -----------------------------------------------------------------------
  // Do not update pCommDCB unless it is being initialized for the first 
  // time.
  // -----------------------------------------------------------------------
  if ( !pCommDCB )
    pCommDCB =  &( pExtData->CommDCB );

  return ( 0 );      // Local memory of EXTDATA successfully allocated.
} 




