/*
***************************************************************************
**              ROCKWELL CONFIDENTIAL AND PROPRIETARY
**        This source is the sole property of ROCKWELL Inc.
**    Reproduction or utilization of this source in whole or in
**  part is forbidden without the written consent of ROCKWELL Inc.
***************************************************************************
**                 (c) Copyright ROCKWELL, INC. 1991
**                       All Rights Reserved
***************************************************************************
*/

/*
**      Last PVCS update: $ $
*/

/*
**      TITLE:   RIF.H
**
**      FACILITY:       
**
**      MODULE DESCRIPTION:
**
**      
**
*/

/*
**
**      Revision History:
**
**              $Log:   D:/api_lib/rif.h_v  $
   
      Rev 1.0   12 Aug 1992 20:07:18   Robert Chen
   Initial revision.
   
***************************************************************************/

//
// RIFF file IO 
//
// Notes:
// - Only certain MMIOINFO fields are supported since custom I/O procedures
//   are not supported.
// - Logging options and output format must be devised.

// --------------------------------------------------------------------------
// Include atomic data types (if being compiled for DOS or Windows w/o 
// WINDOWS.H):
// --------------------------------------------------------------------------
#if defined( VF_DOS ) || !defined( LPSTR )
  typedef int	          BOOL;
  typedef unsigned char	BYTE;
  typedef unsigned int	WORD;
  typedef unsigned long	DWORD;
  typedef char near	    *PSTR;
  typedef char near	    *NPSTR;
  typedef char far	    *LPSTR;
  typedef BYTE near	    *PBYTE;
  typedef BYTE far	    *LPBYTE;
  typedef int near	    *PINT;
  typedef int far 	    *LPINT;
  typedef WORD near	    *PWORD;
  typedef WORD far	    *LPWORD;
  typedef long near	    *PLONG;
  typedef long far	    *LPLONG;
  typedef DWORD near	  *PDWORD;
  typedef DWORD far	    *LPDWORD;
  typedef void far	    *LPVOID;
  typedef void near     *PVOID;

  #define NEAR          near
  #define LONG          long
  #define VOID          void
  #if !defined( PASCAL )
    #define PASCAL      pascal
  #endif
  #if !defined( FAR )
    #define FAR         
  #endif
  #if !defined( TRUE )
    #define TRUE  1
    #define FALSE 0
  #endif

#elif defined( VF_SWIN )
  typedef void near     *PVOID;
#elif defined( VF_MWIN )
  typedef void near     *PVOID;
#endif // ( VF_DOS )

// --------------------------------------------------------------------------
// Include multimedia data types:
// --------------------------------------------------------------------------
#if !defined( VF_MWIN )
  #define WAVE_FORMAT_PCM 1

  #if defined( VF_SWIN )
    typedef LPSTR HPSTR;
  #else
    typedef void near *HPSTR;
  #endif // ( VF_SWIN )

  typedef int   HMMIO;
  typedef DWORD FOURCC;
  typedef int ( FAR PASCAL *LPMMIOPROC )( );

  #define MMIO_FHOPEN           1
  #define MMIO_OUTOFMEMORY      2
  #define MMIOERR_CANNOTWRITE   3
  #define MMIOERR_CANNOTSEEK    4
  #define MMIOERR_CHUNKNOTFOUND 5
  #define MMIO_CREATERIFF       6
  #define MMIO_CREATELIST       7
  #define MMIO_TOUPPER          9

  #define MMIO_READ         0x00000000L // (00000000 0000????) 
  #define MMIO_WRITE        0x00000001L // (00000000 0000????)
  #define MMIO_READWRITE    0x00000002L // (00000000 0000????) 
  #define MMIO_DELETE       0x00000003L // (00000000 0000????)

  #define MMIO_COMPAT       0x00000010L // (00000000 000?0000)
  #define MMIO_EXCLUSIVE    0x00000020L // (00000000 00?00000)
  #define MMIO_DENYWRITE    0x00000040L // (00000000 0?000000) 
  #define MMIO_DENYREAD     0x00000080L // (00000000 ?0000000) 
  #define MMIO_DENYNONE     0x00000100L // (0000000? 00000000) 

  #define MMIO_CREATE       0x00000200L // (000000?0 00000000)
  #define MMIO_ALLOCBUF     0x00000400L // (00000?00 00000000)

  #define MMIO_FINDCHUNK    0x0001      // (00000000 0000????)
  #define MMIO_FINDRIFF     0x0002      // (00000000 0000????)
  #define MMIO_FINDLIST     0x0003      // (00000000 0000????)

  #define MMIO_DIRTY        0x00000001L // (00000000 0000000?)

  #define mmioFOURCC(ch0,ch1,ch2,ch3) \
        ((FOURCC)((((DWORD)(((WORD)ch0)<<8)|((WORD)ch1))<<16)| \
                   ((DWORD)(((WORD)ch2)<<8)|((WORD)ch3))))

  // ------------------------------------------------------------------------
  // Non Microsoft error codes:
  // ------------------------------------------------------------------------
  enum VF_ERRORCODES
  {
    VF_NOTSUPPORTED = -32767, // feature of MS RIFF File I/O not supported
    VF_BADPARAMETER,          // parameter not properly specified
    VF_DOSWRITE,              // DOS error occured while writing a file
    VF_DOSREAD,               // DOS error occured while reading a file
    VF_DOSOPEN                // DOS error occured while openning a file
  };

  typedef struct _MMIOINFO
  {
    DWORD      dwFlags;    // Options, including how a file was opened
    FOURCC     fccIOProc;  // The four-character code of file I/O proc
    LPMMIOPROC pIOProc;    // The address of the file's I/O procedure
    WORD       wErrorRet;  // The extended error value 
    WORD       wReserved;  // Reserved for future use.
    LONG       cchBuffer;  // The size of the file I/O's buffer in bytes
    HPSTR      pchBuffer;  // The address of the file I/O's buffer
    HPSTR      pchNext;    // A huge pointer to the next I/O buffer location
    HPSTR      pchEndRead; // A pointer to a location one byte past last for reading
    HPSTR      pchEndWrite;// A pointer to a location one byte past last for writting
    LONG       lBufOffset; // Reserved for internal use by MMIO functions.
    LONG       lDiskOffset;// Offset in bytes from the beginning of the file
    DWORD      adwInfo[4]; // General state information field
    DWORD      dwReserved1;// Reserved for internal use by MMIO functions.
    DWORD      dwReserved2;// Reserved for internal use by MMIO functions.
    HMMIO      hmmio;      // The MMIO handle to the open file.
  } MMIOINFO;

  typedef MMIOINFO far *LPMMIOINFO;

  typedef struct _MMCKINFO
  {
    FOURCC ckid;         // chunk ID
    DWORD  cksize;       // chunk size
    FOURCC fccType;      // form type or list type

    // Additions of new fields may begin here:
    DWORD  dwDataOffset; // offset of data portion of chunk
    DWORD  dwFlags;      // flags
  } MMCKINFO;

  typedef MMCKINFO far  *LPMMCKINFO;
  typedef MMCKINFO near *NPMMCKINFO;

  // ------------------------------------------------------------------------
  // Size of chunk headers.
  // ------------------------------------------------------------------------
  #define SIZEOF_RCKHEADER ( sizeof( FOURCC ) + sizeof( DWORD ) + sizeof( FOURCC ) )
  #define SIZEOF_SCKHEADER ( sizeof( FOURCC ) + sizeof( DWORD ) )
  #define SIZEOF_RCKFRMLST ( sizeof( FOURCC ) )

  typedef struct _WAVEFORMAT
  {
    WORD   formatTag;       // format category (How can Rockwell ADPCM be registered?)
    WORD   nChannels;       // no. channels
    DWORD  nSamplesPerSec;  // sampling rate
    DWORD  nAvgBytesPerSec; // for buffering
    WORD   nBlockAlign;     // block alignments
  } WAVEFORMAT;

  typedef WAVEFORMAT far *LPWAVEFORMAT;

  // --------------------------------------------------------------------------
  // Audio file format used for uncompressed data in the Microsoft standard.
  // --------------------------------------------------------------------------
  typedef struct _PCMWAVEFORMAT
  {
    WORD wBitsPerSample;
  } PCMWAVEFORMAT;

  // --------------------------------------------------------------------------
  // Basic File I/O:
  // --------------------------------------------------------------------------
  HMMIO mmioOpen( LPSTR, LPMMIOINFO, DWORD );
  WORD  mmioClose( HMMIO, WORD );
  LONG  mmioWrite( HMMIO, HPSTR, WORD );
  LONG  mmioRead( HMMIO, HPSTR, WORD );
  LONG  mmioSeek( HMMIO, LONG, int );
  
  // --------------------------------------------------------------------------
  // I/O Buffer Control Functions:
  // --------------------------------------------------------------------------
  WORD   mmioFlush( HMMIO, WORD );
  WORD   mmioSetBuffer( HMMIO, LPSTR, LONG, WORD );

  // --------------------------------------------------------------------------
  // RIFF File Navigation Functions:
  // --------------------------------------------------------------------------
  WORD   mmioAscend( HMMIO, LPMMCKINFO, WORD );
  WORD   mmioDescend( HMMIO, LPMMCKINFO, LPMMCKINFO, WORD );
  WORD   mmioCreateChunk( HMMIO, LPMMCKINFO, WORD );
  FOURCC mmioStringToFOURCC( LPSTR, WORD );
#endif // (VF_MWIN)

// ==========================================================================
// Universal Manifest Constants and Data Structures:
// ==========================================================================

#define WAVE_FORMAT_RADPCM 2 // format category for Rockwell ADPCM
#define LEN_CMIDSTR       41 // length of compression identifier string
#define SIZEOF_MMIOCKID ( sizeof( FOURCC ) )
#define RIFF_FOURCC ( mmioFOURCC( 'F', 'F', 'I', 'R' ) )
#define LIST_FOURCC ( mmioFOURCC( 'T', 'S', 'I', 'L' ) )

// --------------------------------------------------------------------------
// Audio file format extensions used for Rockwell proprietary compressed 
// data, Rockwell ADPCM.
// --------------------------------------------------------------------------
typedef struct _RADPCMWAVEFORMAT
{
  WORD  wBitsPerSample;
  WORD  wSilenceDeletion;
  char  szCompIdStr[ LEN_CMIDSTR ];
} RADPCMWAVEFORMAT;

// --------------------------------------------------------------------------
// Audio file format fields which must be defined by the caller of the
// top level open function mmioOpenVoice.
// --------------------------------------------------------------------------
typedef struct _RADPCMVOICE
{
  // ------------------------------------------------------------------------
  // These fields are used internally to navigate the chunks.  Program
  // modifications may occur in this data structure from this point only.
  // ------------------------------------------------------------------------
  MMCKINFO ckParent;
  MMCKINFO ckFmt;
  MMCKINFO ckData;

  HMMIO hFileHandle;

  // ------------------------------------------------------------------------
  // Modifications to this data structure should not occur between the 
  // following two fields.
  // ------------------------------------------------------------------------
  WAVEFORMAT wf;           
  union _WAVEFORMATSPEC
  {                        // nChannels, nSamplesPerSec, formatTag:
    RADPCMWAVEFORMAT rwf;  // user maintained Rockwell specific formatting
    PCMWAVEFORMAT pwf;     // WAVE file format supported by Microsoft
  };
} VF;

#define SIZEOF_RFMTDATA ( sizeof( WAVEFORMAT ) + sizeof( RADPCMWAVEFORMAT ) )
#define SIZEOF_MFMTDATA ( sizeof( WAVEFORMAT ) + sizeof( PCMWAVEFORMAT ) )
#define SIZEOF_RDATA    ( sizeof( RADPCMWAVEFORMAT ) )
#define SIZEOF_MDATA    ( sizeof( PCMWAVEFORMAT ) )
#define SIZEOF_WAVEDATA ( sizeof( WAVEFORMAT ) )

typedef VF far  *LPVF;
typedef VF near *NPVF;

// --------------------------------------------------------------------------
// Top level user functions for accessing a standard Rockwell ADPCM Voice
// file.
// --------------------------------------------------------------------------
int  FAR PASCAL VF_OpenVoice( register LPVF, LPSTR, DWORD );
int  FAR PASCAL VF_CloseVoice( register LPVF );
WORD FAR PASCAL VF_WriteVoice( register LPVF, HPSTR, WORD );
WORD FAR PASCAL VF_ReadVoice( register LPVF, HPSTR, WORD );

// --------------------------------------------------------------------------
// Utility functions to be used in printing formatted RIF header information.
// --------------------------------------------------------------------------
void PrintFormatSubchunk( NPVF, char * );
void PrintSubchunkHead( NPMMCKINFO, char * );
void PrintHexStr( PVOID, int );
BOOL IsRIFFile( LPSTR );


